from os.path import join
from dvha.paths import TG263_CSV, PREF_DIR


class ROIMapGenerator:
    """Class to interact with the TG263 table and generate compliant ROI Name Maps"""

    def __init__(self):
        # Load TG263 table
        with open(TG263_CSV, "r") as doc:
            keys = [key.strip() for key in doc.readline().split(",")]
            self.tg_263 = {key: [] for key in keys}
            for line in doc:
                if "xx" not in line:  # ignore the rows with generic expansions
                    for col, value in enumerate(line.split(",")):
                        if value == "":
                            value = "None"
                        self.tg_263[keys[col]].append(
                            value.strip().replace("^", ",")
                        )
            self.keys = keys
            self.keys.append(self.keys.pop(0))  # Move Target Type to the end

        self.key_map = {key: key for key in keys}

    def __call__(
        self,
        map_file_name,
        body_sites=None,
        data_filter=None,
        roi_uid_type="primary",
    ):
        """
        Create a new ROI map file based on TG263
        :param map_file_name: save the file in PREF_DIR with this name
        :type map_file_name: str
        :param body_sites: automatically create a data_filter based on these body sites
        :type body_sites: list
        :param data_filter: used to write a subset of the TG263 data
        :type data_filter: dict
        :param roi_uid_type: either 'primary', 'reverse', or 'fmaid'
        :type roi_uid_type: str
        :return: file_path of new map file
        """
        if body_sites is not None:
            if type(body_sites) is not list:
                body_sites = [body_sites]
            data_filter = {"Anatomic Group": body_sites}
        data = (
            self.tg_263
            if data_filter is None
            else self.get_filtered_data(data_filter)
        )

        lookup = {
            "primary": "TG263-Primary Name",
            "reverse": "TG-263-Reverse Order Name",
            "fmaid": "FMAID",
        }
        roi_uids = self.get_unique_values(lookup[roi_uid_type], data)

        file_path = join(PREF_DIR, map_file_name)
        with open(file_path, "w") as doc:
            for roi_uid in roi_uids:
                doc.write(": ".join([roi_uid] * 3) + "\n")
        return file_path

    def prep_data_for_roi_map_gui(self):
        """ """
        ignored = [
            c
            for c in self.keys
            if "Reverse" in c or "Character" in c or "FMAID" in c
        ]
        for c in ignored:
            self.drop_column(c)

        str_map = {"Category": "Cat.", "Anatomic": "Anat.", "TG263-": ""}
        for c in self.keys:
            if any([key in c for key in list(str_map)]):
                new_key = c
                for key, value in str_map.items():
                    new_key = new_key.replace(key, value)
                self.key_map[c] = new_key
                key_index = self.keys.index(c)
                self.tg_263[new_key] = self.drop_column(c, return_data=True)
                self.keys.insert(key_index, new_key)

    ##########################################################
    # Generalized Tools
    ##########################################################
    def get_filtered_data(self, data_filter):
        """Get TG263 data with a filter

        Parameters
        ----------
        data_filter : dict
            column: list_allowed_values}

        Returns
        -------
        type
            subset of tg_263 with the data_filter applied

        """

        for key in list(data_filter):
            if (
                not isinstance(data_filter[key], list)
                and data_filter[key].lower() == "all"
            ):
                data_filter.pop(key)

        data = {key: [] for key in self.keys}
        for row in range(len(self.tg_263[self.keys[0]])):
            is_included = [
                self.tg_263[col][row] in data_filter[col]
                for col in list(data_filter)
            ]
            if all(is_included):
                for c in self.keys:
                    data[c].append(self.tg_263[c][row])
        return data

    def get_unique_values(self, column, data=None):
        """

        Parameters
        ----------
        column :
            TG263 column
        data :
            optionally provide filtered data, default is entire TG263 table

        Returns
        -------
        type
            list of unique values for the provided column

        """
        data = self.tg_263 if data is None else data
        return sorted([value for value in set(data[column]) if value])

    def get_value_from_uid(self, roi_uid, output_type, reverse_name=False):
        """

        Parameters
        ----------
        roi_uid : str
            either a primary name or FMAID (auto-detects)
        output_type : str
            column name of output value
        reverse_name : bool
            roi_uid is assumed to be reverse order name if True (Default value = False)

        Returns
        -------
        type
            another column value of the provided roi_uid

        """
        input_type = (
            "FMAID"
            if roi_uid.isdigit()
            else ["TG-263-Reverse Order Name", "TG263-Primary Name"][
                reverse_name
            ]
        )
        return self._get_value_from_uid(
            roi_uid,
            input_type=self.key_map[input_type],
            output_type=self.key_map[output_type],
        )

    def _get_value_from_uid(self, input_value, input_type, output_type):
        """Generic function to look up another column with a given input value and type

        Parameters
        ----------
        input_value :

        input_type :

        output_type :


        Returns
        -------

        """
        if input_value in self.tg_263[input_type]:
            index = self.tg_263[input_type].index(input_value)
            return self.tg_263[output_type][index]

    def drop_column(self, column, return_data=False):
        """Remove a column from tg_263 data and update keys

        Parameters
        ----------
        column :

        return_data :
             (Default value = False)

        Returns
        -------

        """
        if column in self.keys and column in list(self.tg_263):
            self.keys.pop(self.keys.index(column))
            popped_data = self.tg_263.pop(column)
        if return_data:
            return popped_data

    ##########################################################
    # Properties for coding ease
    ##########################################################
    @property
    def target_types(self):
        """ """
        return self.get_unique_values(self.key_map["Target Type"])

    @property
    def anatomic_groups(self):
        """ """
        return self.get_unique_values(self.key_map["Anatomic Group"])

    @property
    def major_categories(self):
        """ """
        return self.get_unique_values(self.key_map["Major Category"])

    @property
    def minor_categories(self):
        """ """
        return self.get_unique_values(self.key_map["Minor Category"])

    @property
    def primary_names(self):
        """ """
        return self.get_unique_values(self.key_map["TG263-Primary Name"])

    @property
    def reverse_order_primary_names(self):
        """ """
        return self.get_unique_values(
            self.key_map["TG-263-Reverse Order Name"]
        )

    @property
    def fmaids(self):
        """ """
        return self.get_unique_values(self.key_map["FMAID"])

    def get_primary_name(self, fmaid):
        """

        Parameters
        ----------
        fmaid :


        Returns
        -------

        """
        return self._get_value_from_uid(
            fmaid, input_type="FMAID", output_type="TG263-Primary Name"
        )

    def get_fmaid(self, primary_name):
        """

        Parameters
        ----------
        primary_name :


        Returns
        -------

        """
        return self._get_value_from_uid(
            primary_name, input_type="TG263-Primary Name", output_type="FMAID"
        )

    def get_target_type(self, roi_uid):
        """

        Parameters
        ----------
        roi_uid :


        Returns
        -------

        """
        return self.get_value_from_uid(roi_uid, output_type="Target Type")

    def get_major_category(self, roi_uid):
        """

        Parameters
        ----------
        roi_uid :


        Returns
        -------

        """
        return self.get_value_from_uid(roi_uid, output_type="Major Category")

    def get_minor_category(self, roi_uid):
        """

        Parameters
        ----------
        roi_uid :


        Returns
        -------

        """
        return self.get_value_from_uid(roi_uid, output_type="Minor Category")

    def get_anatomic_group(self, roi_uid):
        """

        Parameters
        ----------
        roi_uid :


        Returns
        -------

        """
        return self.get_value_from_uid(roi_uid, output_type="Anatomic Group")

    def get_reverse_order_name(self, roi_uid):
        """

        Parameters
        ----------
        roi_uid :


        Returns
        -------

        """
        return self.get_value_from_uid(
            roi_uid, output_type="TG-263-Reverse Order Name"
        )

    def get_description(self, roi_uid):
        """

        Parameters
        ----------
        roi_uid :


        Returns
        -------

        """
        return self.get_value_from_uid(roi_uid, output_type="Description")



# https://stackoverflow.com/questions/28509629/work-with-ctrl-c-and-ctrl-v-to-copy-and-paste-into-a-wx-grid-in-wxpython
# Accessed July 12, 2019: answered by Sinan Çetinkaya
import wx.grid
import wx


class Spreadsheet(wx.grid.Grid):
    def __init__(self, parent):
        wx.grid.Grid.__init__(
            self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, 0
        )
        self.Bind(wx.EVT_KEY_DOWN, self.on_key)
        self.Bind(wx.grid.EVT_GRID_CELL_CHANGING, self.on_change)
        self.Bind(
            wx.grid.EVT_GRID_LABEL_RIGHT_CLICK, self.on_label_right_click
        )
        self.Bind(wx.grid.EVT_GRID_CELL_RIGHT_CLICK, self.on_cell_right_click)
        self.selected_rows = []
        self.selected_cols = []
        self.history = []

    def get_col_headers(self):
        return [
            self.GetColLabelValue(col) for col in range(self.GetNumberCols())
        ]

    def get_table(self):
        for row in range(self.GetNumberRows()):
            result = {}
            for col, header in enumerate(self.get_col_headers()):
                result[header] = self.GetCellValue(row, col)
            yield result

    def add_rows(self, event):
        for row in self.selected_rows:
            self.InsertRows(row)
        self.add_history({"type": "add_rows", "rows": self.selected_rows})

    def add_cols(self, event):
        for col in self.selected_cols:
            self.InsertCols(col)
        self.add_history({"type": "add_cols", "cols": self.selected_cols})

    def delete_rows(self, event):
        self.delete_row_or_col(event, "rows")

    def delete_cols(self, event):
        self.delete_row_or_col(event, "cols")

    def delete_row_or_col(self, event, del_type):
        action = [self.DeleteRows, self.DeleteCols][del_type == "cols"]
        selected = [self.selected_rows, self.selected_cols][del_type == "cols"]
        self.delete(event)
        data = []
        for element in reversed(selected):
            data.append(
                (
                    element,
                    {  # More attributes can be added
                        "label": self.GetColLabelValue(element),
                        "size": self.GetColSize(element),
                    },
                )
            )
            action(element)
        self.add_history({"type": "delete_%s" % del_type, "cols": data})

    def on_cell_right_click(self, event):
        menus = [
            (wx.NewId(), "Cut", self.cut),
            (wx.NewId(), "Copy", self.copy),
            (wx.NewId(), "Paste", self.paste),
        ]
        popup_menu = wx.Menu()
        for menu in menus:
            if menu is None:
                popup_menu.AppendSeparator()
                continue
            popup_menu.Append(menu[0], menu[1])
            self.Bind(wx.EVT_MENU, menu[2], id=menu[0])

        self.PopupMenu(popup_menu, event.GetPosition())
        popup_menu.Destroy()
        return

    def on_label_right_click(self, event):
        menus = [
            (wx.NewId(), "Cut", self.cut),
            (wx.NewId(), "Copy", self.copy),
            (wx.NewId(), "Paste", self.paste),
            None,
        ]

        # Select if right clicked row or column is not in selection
        if event.GetRow() > -1:
            # if not self.IsInSelection(row=event.GetRow(), col=1):
            #     self.SelectRow(event.GetRow())
            # self.selected_rows = self.GetSelectedRows()
            # menus += [(wx.NewId(), "Add row", self.add_rows)]
            # menus += [(wx.NewId(), "Delete row", self.delete_rows)]
            pass
        elif event.GetCol() > -1:
            if not self.IsInSelection(row=1, col=event.GetCol()):
                self.SelectCol(event.GetCol())
            self.selected_cols = self.GetSelectedCols()
            menus += [(wx.NewId(), "Add column", self.add_cols)]
            # menus += [(wx.NewId(), "Delete column", self.delete_cols)]
        else:
            return

        popup_menu = wx.Menu()
        for menu in menus:
            if menu is None:
                popup_menu.AppendSeparator()
                continue
            popup_menu.Append(menu[0], menu[1])
            self.Bind(wx.EVT_MENU, menu[2], id=menu[0])

        self.PopupMenu(popup_menu, event.GetPosition())
        popup_menu.Destroy()
        return

    def on_change(self, event):
        cell = event.GetEventObject()
        row = cell.GetGridCursorRow()
        col = cell.GetGridCursorCol()
        attribute = {"value": self.GetCellValue(row, col)}
        self.add_history({"type": "change", "cells": [(row, col, attribute)]})

    def add_history(self, change):
        self.history.append(change)

    def undo(self):
        if not len(self.history):
            return

        action = self.history.pop()
        if action["type"] == "change" or action["type"] == "delete":
            for row, col, attribute in action["cells"]:
                self.SetCellValue(row, col, attribute["value"])
                if action["type"] == "delete":
                    self.SetCellAlignment(
                        row, col, *attribute["alignment"]
                    )  # *attribute["alignment"] > horiz, vert

        elif action["type"] == "delete_rows":
            for row, attribute in reversed(action["rows"]):
                self.InsertRows(row)
                self.SetRowLabelValue(row, attribute["label"])
                self.SetRowSize(row, attribute["size"])

        elif action["type"] == "delete_cols":
            for col, attribute in reversed(action["cols"]):
                self.InsertCols(col)
                self.SetColLabelValue(col, attribute["label"])
                self.SetColSize(col, attribute["size"])

        elif action["type"] == "add_rows":
            for row in reversed(action["rows"]):
                self.DeleteRows(row)

        elif action["type"] == "add_cols":
            for col in reversed(action["cols"]):
                self.DeleteCols(col)
        else:
            return

    def on_key(self, event):
        """
        Handles all key events.
        """
        # print(event.GetKeyCode())
        # Ctrl+C or Ctrl+Insert
        if event.ControlDown() and event.GetKeyCode() in [67, 322]:
            self.copy(event)

        # Ctrl+V
        elif event.ControlDown() and event.GetKeyCode() == 86:
            self.paste(event)

        # DEL
        elif event.GetKeyCode() == 127:
            self.delete(event)

        # Ctrl+A
        elif event.ControlDown() and event.GetKeyCode() == 65:
            self.SelectAll()

        # Ctrl+Z
        elif event.ControlDown() and event.GetKeyCode() == 90:
            self.undo()

        # Ctrl+X
        elif event.ControlDown() and event.GetKeyCode() == 88:
            # Call delete method
            self.cut(event)

        # Ctrl+V or Shift + Insert
        elif (event.ControlDown() and event.GetKeyCode() == 67) or (
            event.ShiftDown() and event.GetKeyCode() == 322
        ):
            self.paste(event)

        else:
            event.Skip()

    def get_selection(self):
        """
        Returns selected range's start_row, start_col, end_row, end_col
        If there is no selection, returns selected cell's start_row=end_row, start_col=end_col
        """
        if not len(self.GetSelectionBlockTopLeft()):
            selected_columns = self.GetSelectedCols()
            selected_rows = self.GetSelectedRows()
            if selected_columns:
                start_col = selected_columns[0]
                end_col = selected_columns[-1]
                start_row = 0
                end_row = self.GetNumberRows() - 1
            elif selected_rows:
                start_row = selected_rows[0]
                end_row = selected_rows[-1]
                start_col = 0
                end_col = self.GetNumberCols() - 1
            else:
                start_row = end_row = self.GetGridCursorRow()
                start_col = end_col = self.GetGridCursorCol()
        elif len(self.GetSelectionBlockTopLeft()) > 1:
            wx.MessageBox("Multiple selections are not supported", "Warning")
            return []
        else:
            start_row, start_col = self.GetSelectionBlockTopLeft()[0]
            end_row, end_col = self.GetSelectionBlockBottomRight()[0]

        return [start_row, start_col, end_row, end_col]

    def get_selected_cells(self):
        # returns a list of selected cells
        selection = self.get_selection()
        if not selection:
            return

        start_row, start_col, end_row, end_col = selection
        for row in range(start_row, end_row + 1):
            for col in range(start_col, end_col + 1):
                yield [row, col]

    def copy(self, event):
        """
        Copies range of selected cells to clipboard.
        """

        selection = self.get_selection()
        if not selection:
            return []
        start_row, start_col, end_row, end_col = selection

        data = u""

        rows = range(start_row, end_row + 1)
        for row in rows:
            columns = range(start_col, end_col + 1)
            for idx, column in enumerate(columns, 1):
                if idx == len(columns):
                    # if we are at the last cell of the row, add new line instead
                    data += self.GetCellValue(row, column) + "\n"
                else:
                    data += self.GetCellValue(row, column) + "\t"

        text_data_object = wx.TextDataObject()
        text_data_object.SetText(data)

        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(text_data_object)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox("Can't open the clipboard", "Warning")

    def paste(self, event):
        if not wx.TheClipboard.Open():
            wx.MessageBox("Can't open the clipboard", "Warning")
            return False

        clipboard = wx.TextDataObject()
        wx.TheClipboard.GetData(clipboard)
        wx.TheClipboard.Close()
        data = clipboard.GetText()
        if data[-1] == "\n":
            data = data[:-1]

        try:
            cells = self.get_selected_cells()
            cell = next(cells)
        except StopIteration:
            return False

        start_row = end_row = cell[0]
        start_col = end_col = cell[1]
        max_row = self.GetNumberRows()
        max_col = self.GetNumberCols()

        history = []
        out_of_range = False

        for row, line in enumerate(data.split("\n")):
            target_row = start_row + row
            if not (0 <= target_row < max_row):
                out_of_range = True
                break

            if target_row > end_row:
                end_row = target_row

            for col, value in enumerate(line.split("\t")):
                target_col = start_col + col
                if not (0 <= target_col < max_col):
                    out_of_range = True
                    break

                if target_col > end_col:
                    end_col = target_col

                # save previous value of the cell for undo
                history.append(
                    [
                        target_row,
                        target_col,
                        {"value": self.GetCellValue(target_row, target_col)},
                    ]
                )

                self.SetCellValue(target_row, target_col, value)

        self.SelectBlock(
            start_row, start_col, end_row, end_col
        )  # select pasted range
        if out_of_range:
            wx.MessageBox("Pasted data is out of Grid range", "Warning")

        self.add_history({"type": "change", "cells": history})

    def delete(self, event):
        cells = []
        for row, col in self.get_selected_cells():
            attributes = {
                "value": self.GetCellValue(row, col),
                "alignment": self.GetCellAlignment(row, col),
            }
            cells.append((row, col, attributes))
            self.SetCellValue(row, col, "")

        self.add_history({"type": "delete", "cells": cells})

    def cut(self, event):
        self.copy(event)
        self.delete(event)


# class MyFrame(wx.Frame):
#     def __init__(self, parent, ID, title, pos=wx.DefaultPosition, size=wx.Size(800, 400), style=wx.DEFAULT_FRAME_STYLE):
#         wx.Frame.__init__(self, parent, ID, title, pos, size, style)
#         agrid = Spreadsheet(self)
#         agrid.CreateGrid(7, 7)
#         for count in range(3):
#             for count2 in range(3):
#                 agrid.SetCellValue(count, count2, str(count + count2))


# class MyApp(wx.App):
#     def OnInit(self):
#         frame = MyFrame(None, -1, "A copy and paste grid")
#         frame.Show(True)
#         self.SetTopWindow(frame)
#         return True


# app = MyApp()
# app.MainLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# db.sql_columns.py
"""
These objects are largely used for designing queries in the GUI to easily
connect UI friendly variable names with their associated SQL tables column
names
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

# This is a maps categorical data type selections to SQL columns and SQL tables
categorical = {
    "ROI Institutional Category": {
        "var_name": "institutional_roi",
        "table": "DVHs",
    },
    "ROI Physician Category": {"var_name": "physician_roi", "table": "DVHs"},
    "ROI Name": {"var_name": "roi_name", "table": "DVHs"},
    "ROI Type": {"var_name": "roi_type", "table": "DVHs"},
    "Beam Type": {"var_name": "beam_type", "table": "Beams"},
    "Collimator Rotation Direction": {
        "var_name": "collimator_rot_dir",
        "table": "Beams",
    },
    "Couch Rotation Direction": {
        "var_name": "couch_rot_dir",
        "table": "Beams",
    },
    "Dose Grid Resolution": {"var_name": "dose_grid_res", "table": "Plans"},
    "Gantry Rotation Direction": {
        "var_name": "gantry_rot_dir",
        "table": "Beams",
    },
    "Radiation Type": {"var_name": "radiation_type", "table": "Beams"},
    "Patient Orientation": {
        "var_name": "patient_orientation",
        "table": "Plans",
    },
    "Patient Sex": {"var_name": "patient_sex", "table": "Plans"},
    "Physician": {"var_name": "physician", "table": "Plans"},
    "Tx Modality": {"var_name": "tx_modality", "table": "Plans"},
    "Tx Site": {"var_name": "tx_site", "table": "Plans"},
    "Normalization": {"var_name": "normalization_method", "table": "Rxs"},
    "Treatment Machine": {"var_name": "treatment_machine", "table": "Beams"},
    "Heterogeneity Correction": {
        "var_name": "heterogeneity_correction",
        "table": "Plans",
    },
    "Scan Mode": {"var_name": "scan_mode", "table": "Beams"},
    "MRN": {"var_name": "mrn", "table": "Plans"},
    "UID": {"var_name": "study_instance_uid", "table": "Plans"},
    "Baseline": {"var_name": "baseline", "table": "Plans"},
    "Protocol": {"var_name": "protocol", "table": "Plans"},
}

# This is a maps quantitative data type selections to SQL columns and SQL
# tables, and the bokeh source
numerical = {
    "Age": {"var_name": "age", "table": "Plans", "units": ""},
    "Beam Energy Min": {
        "var_name": "beam_energy_min",
        "table": "Beams",
        "units": "",
    },
    "Beam Energy Max": {
        "var_name": "beam_energy_max",
        "table": "Beams",
        "units": "",
    },
    "Birth Date": {"var_name": "birth_date", "table": "Plans", "units": ""},
    "Planned Fractions": {"var_name": "fxs", "table": "Plans", "units": ""},
    "Rx Dose": {"var_name": "rx_dose", "table": "Plans", "units": "Gy"},
    "Rx Isodose": {"var_name": "rx_percent", "table": "Rxs", "units": "%"},
    "Simulation Date": {
        "var_name": "sim_study_date",
        "table": "Plans",
        "units": "",
    },
    "Total Plan MU": {"var_name": "total_mu", "table": "Plans", "units": "MU"},
    "Fraction Dose": {"var_name": "fx_dose", "table": "Rxs", "units": "Gy"},
    "Beam Dose": {"var_name": "beam_dose", "table": "Beams", "units": "Gy"},
    "Beam MU": {"var_name": "beam_mu", "table": "Beams", "units": ""},
    "Control Point Count": {
        "var_name": "control_point_count",
        "table": "Beams",
        "units": "",
    },
    "SSD": {"var_name": "ssd", "table": "Beams", "units": "cm"},
    "ROI Min Dose": {"var_name": "min_dose", "table": "DVHs", "units": "Gy"},
    "ROI Mean Dose": {"var_name": "mean_dose", "table": "DVHs", "units": "Gy"},
    "ROI Max Dose": {"var_name": "max_dose", "table": "DVHs", "units": "Gy"},
    "ROI Volume": {"var_name": "volume", "table": "DVHs", "units": "cm³"},
    "ROI Surface Area": {
        "var_name": "surface_area",
        "table": "DVHs",
        "units": "cm²",
    },
    "ROI Spread X": {"var_name": "spread_x", "table": "DVHs", "units": "cm"},
    "ROI Spread Y": {"var_name": "spread_y", "table": "DVHs", "units": "cm"},
    "ROI Spread Z": {"var_name": "spread_z", "table": "DVHs", "units": "cm"},
    "PTV Distance (Min)": {
        "var_name": "dist_to_ptv_min",
        "table": "DVHs",
        "units": "cm",
    },
    "PTV Distance (Mean)": {
        "var_name": "dist_to_ptv_mean",
        "table": "DVHs",
        "units": "cm",
    },
    "PTV Distance (Median)": {
        "var_name": "dist_to_ptv_median",
        "table": "DVHs",
        "units": "cm",
    },
    "PTV Distance (Max)": {
        "var_name": "dist_to_ptv_max",
        "table": "DVHs",
        "units": "cm",
    },
    "PTV Distance (Centroids)": {
        "var_name": "dist_to_ptv_centroids",
        "table": "DVHs",
        "units": "cm",
    },
    "PTV Overlap": {
        "var_name": "ptv_overlap",
        "table": "DVHs",
        "units": "cm³",
    },
    "Scan Spots": {
        "var_name": "scan_spot_count",
        "table": "Beams",
        "units": "",
    },
    "Beam MU per deg": {
        "var_name": "beam_mu_per_deg",
        "table": "Beams",
        "units": "",
    },
    "Beam MU per control point": {
        "var_name": "beam_mu_per_cp",
        "table": "Beams",
        "units": "",
    },
    "ROI Cross-Section Max": {
        "var_name": "cross_section_max",
        "table": "DVHs",
        "units": "cm²",
    },
    "ROI Cross-Section Median": {
        "var_name": "cross_section_median",
        "table": "DVHs",
        "units": "cm²",
    },
    "Toxicity Grade": {
        "var_name": "toxicity_grade",
        "table": "DVHs",
        "units": "",
    },
    "ROI Centroid to Isocenter (Min)": {
        "var_name": "centroid_dist_to_iso_min",
        "table": "DVHs",
        "units": "cm",
    },
    "ROI Centroid to Isocenter (Max)": {
        "var_name": "centroid_dist_to_iso_max",
        "table": "DVHs",
        "units": "cm",
    },
    "Plan Complexity": {
        "var_name": "complexity",
        "table": "Plans",
        "units": "",
    },
    "Beam Complexity (Min)": {
        "var_name": "complexity_min",
        "table": "Beams",
        "units": "",
    },
    "Beam Complexity (Mean)": {
        "var_name": "complexity_mean",
        "table": "Beams",
        "units": "",
    },
    "Beam Complexity (Median)": {
        "var_name": "complexity_median",
        "table": "Beams",
        "units": "",
    },
    "Beam Complexity (Max)": {
        "var_name": "complexity_max",
        "table": "Beams",
        "units": "",
    },
    "Beam Area (Min)": {
        "var_name": "area_min",
        "table": "Beams",
        "units": "cm²",
    },
    "Beam Area (Mean)": {
        "var_name": "area_mean",
        "table": "Beams",
        "units": "cm²",
    },
    "Beam Area (Median)": {
        "var_name": "area_median",
        "table": "Beams",
        "units": "cm²",
    },
    "Beam Area (Max)": {
        "var_name": "area_max",
        "table": "Beams",
        "units": "cm²",
    },
    "Beam Perimeter X (Min)": {
        "var_name": "x_perim_min",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter X (Mean)": {
        "var_name": "x_perim_mean",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter X (Median)": {
        "var_name": "x_perim_median",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter X (Max)": {
        "var_name": "x_perim_max",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter Y (Min)": {
        "var_name": "y_perim_min",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter Y (Mean)": {
        "var_name": "y_perim_mean",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter Y (Median)": {
        "var_name": "y_perim_median",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter Y (Max)": {
        "var_name": "y_perim_max",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter (Min)": {
        "var_name": "perim_min",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter (Mean)": {
        "var_name": "perim_mean",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter (Median)": {
        "var_name": "perim_median",
        "table": "Beams",
        "units": "cm",
    },
    "Beam Perimeter (Max)": {
        "var_name": "perim_max",
        "table": "Beams",
        "units": "cm",
    },
    "Fx Group Beam Count": {
        "var_name": "fx_grp_beam_count",
        "table": "Beams",
        "units": "",
    },
    "Control Point MU (Min)": {
        "var_name": "cp_mu_min",
        "table": "Beams",
        "units": "",
    },
    "Control Point MU (Mean)": {
        "var_name": "cp_mu_mean",
        "table": "Beams",
        "units": "",
    },
    "Control Point MU (Median)": {
        "var_name": "cp_mu_median",
        "table": "Beams",
        "units": "",
    },
    "Control Point MU (Max)": {
        "var_name": "cp_mu_max",
        "table": "Beams",
        "units": "",
    },
    "PTV Cross-Section Max": {
        "var_name": "ptv_cross_section_max",
        "table": "Plans",
        "units": "cm²",
    },
    "PTV Cross-Section Median": {
        "var_name": "ptv_cross_section_median",
        "table": "Plans",
        "units": "cm²",
    },
    "PTV Spread X": {
        "var_name": "ptv_spread_x",
        "table": "Plans",
        "units": "cm",
    },
    "PTV Spread Y": {
        "var_name": "ptv_spread_y",
        "table": "Plans",
        "units": "cm",
    },
    "PTV Spread Z": {
        "var_name": "ptv_spread_z",
        "table": "Plans",
        "units": "cm",
    },
    "PTV Surface Area": {
        "var_name": "ptv_surface_area",
        "table": "Plans",
        "units": "cm²",
    },
    "PTV Volume": {"var_name": "ptv_volume", "table": "Plans", "units": "cm³"},
    "PTV Max Dose": {
        "var_name": "ptv_max_dose",
        "table": "Plans",
        "units": "Gy",
    },
    "PTV Min Dose": {
        "var_name": "ptv_min_dose",
        "table": "Plans",
        "units": "Gy",
    },
}

# Removed these variables from UI as they may be overkill?
numerical_detailed = {
    "Collimator Start Angle": {
        "var_name": "collimator_start",
        "table": "Beams",
        "units": "deg",
    },
    "Collimator End Angle": {
        "var_name": "collimator_end",
        "table": "Beams",
        "units": "deg",
    },
    "Collimator Min Angle": {
        "var_name": "collimator_min",
        "table": "Beams",
        "units": "deg",
    },
    "Collimator Max Angle": {
        "var_name": "collimator_max",
        "table": "Beams",
        "units": "deg",
    },
    "Collimator Range": {
        "var_name": "collimator_range",
        "table": "Beams",
        "units": "deg",
    },
    "Couch Start Angle": {
        "var_name": "couch_start",
        "table": "Beams",
        "units": "deg",
    },
    "Couch End Angle": {
        "var_name": "couch_end",
        "table": "Beams",
        "units": "deg",
    },
    "Couch Min Angle": {
        "var_name": "couch_min",
        "table": "Beams",
        "units": "deg",
    },
    "Couch Max Angle": {
        "var_name": "couch_max",
        "table": "Beams",
        "units": "deg",
    },
    "Couch Range": {
        "var_name": "couch_range",
        "table": "Beams",
        "units": "deg",
    },
    "Gantry Start Angle": {
        "var_name": "gantry_start",
        "table": "Beams",
        "units": "deg",
    },
    "Gantry End Angle": {
        "var_name": "gantry_end",
        "table": "Beams",
        "units": "deg",
    },
    "Gantry Min Angle": {
        "var_name": "gantry_min",
        "table": "Beams",
        "units": "deg",
    },
    "Gantry Max Angle": {
        "var_name": "gantry_max",
        "table": "Beams",
        "units": "deg",
    },
    "Gantry Range": {
        "var_name": "gantry_range",
        "table": "Beams",
        "units": "deg",
    },
}
numerical_detailed.update(numerical)

all_columns = {}
all_columns.update(categorical)
all_columns.update(numerical_detailed)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# main.py
"""
The main file DVH Analytics
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import logging, logging.handlers
import sys, threading, traceback, os

logger = logging.getLogger("dvha")
logger.setLevel(logging.DEBUG)

import wx
from datetime import datetime
import webbrowser
from pubsub import pub
from dvha.db import sql_columns
from dvha.db.sql_to_python import QuerySQL
from dvha.db.sql_connector import echo_sql_db, initialize_db
from dvha.dialogs.main import (
    query_dlg,
    UserSettings,
    About,
    PythonLibraries,
    do_sqlite_backup,
)
from dvha.dialogs.database import SQLSettingsDialog
from dvha.dialogs.export import (
    ExportCSVDialog,
    ExportFigure,
    ExportPGSQLProgressFrame,
)
from dvha.models.import_dicom import ImportDicomFrame
from dvha.models.database_editor import DatabaseEditorFrame
from dvha.models.data_table import DataTable
from dvha.models.plot import PlotStatDVH
from dvha.models.dvh import DVH
from dvha.models.endpoint import EndpointFrame
from dvha.models.queried_data import QueriedDataFrame
from dvha.models.rad_bio import RadBioFrame
from dvha.models.time_series import TimeSeriesFrame
from dvha.models.correlation import CorrelationFrame
from dvha.models.machine_learning import (
    MachineLearningModelViewer,
    ALGORITHMS,
    MachineLearningSetupDlg,
)
from dvha.models.regression import RegressionFrame, LoadMultiVarModelFrame
from dvha.models.control_chart import ControlChartFrame
from dvha.models.roi_map import ROIMapFrame
from dvha.models.stats_data_editor import StatsDataEditor
from dvha.options import Options, DefaultOptions
from dvha.paths import LOGO_PATH, DATA_DIR, ICONS, MODELS_DIR, APP_DIR
from dvha.tools.errors import (
    MemoryErrorDialog,
    PlottingMemoryError,
    ErrorDialog,
)
from dvha.tools.roi_name_manager import DatabaseROIs
from dvha.tools.stats import StatsData, sync_variables_in_stats_data_objects
from dvha.tools.utilities import (
    get_study_instance_uids,
    scale_bitmap,
    is_windows,
    is_linux,
    is_mac,
    get_window_size,
    save_object_to_file,
    load_object_from_file,
    set_msw_background_color,
    initialize_directories,
    set_frame_icon,
    main_is_frozen,
    apply_resolution_limits,
)
from dvha.db.sql_columns import all_columns as sql_column_info


class DVHAMainFrame(wx.Frame):
    def __init__(self, *args, **kwds):

        self.allow_window_size_save = False

        #############################################################################
        # The following block of code for logging adapted from dicompyler
        #############################################################################
        # Initialize logging
        logger = logging.getLogger("dvha")

        # Configure the exception hook to process threads as well
        self.InstallThreadExcepthook()

        # Remap the exception hook so that we can log and display exceptions
        def LogExcepthook(*exc_info):
            # Log the exception
            text = "".join(traceback.format_exception(*exc_info))
            logger.error("Unhandled exception: %s", text)
            pub.sendMessage("logging.exception", msg=text)

        sys.excepthook = LogExcepthook

        # Modify the logging system from pydicom to capture important messages
        pydicom_logger = logging.getLogger("pydicom")
        for l in pydicom_logger.handlers:
            pydicom_logger.removeHandler(l)

        # Add file logger
        logpath = os.path.join(APP_DIR, "logs")
        if not os.path.exists(logpath):
            os.makedirs(logpath)
        self.fh = logging.handlers.RotatingFileHandler(
            os.path.join(logpath, "dvha.log"), maxBytes=524288, backupCount=7
        )
        self.fh.setFormatter(
            logging.Formatter(
                "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
            )
        )
        self.fh.setLevel(logging.WARNING)
        logger.addHandler(self.fh)
        pydicom_logger.addHandler(self.fh)

        # Add console logger if not frozen
        if not main_is_frozen():
            self.ch = logging.StreamHandler()
            self.ch.setFormatter(
                logging.Formatter("%(levelname)s: %(message)s")
            )
            self.ch.setLevel(logging.WARNING)
            logger.addHandler(self.ch)
            pydicom_logger.addHandler(self.ch)
        # Otherwise if frozen, send stdout/stderror to /dev/null since
        # logging the messages seems to cause instability due to recursion
        else:
            devnull = open(os.devnull, "w")
            sys.stdout = devnull
            sys.stderr = devnull
        #############################################################################
        # End logging code block from dicompyler
        #############################################################################

        kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        self.layout_set = False

        self.sizer_dvhs = wx.BoxSizer(wx.VERTICAL)

        set_msw_background_color(
            self
        )  # If windows, change the background color

        self.options = Options()
        self.SetMinSize(self.options.MIN_RESOLUTION_MAIN)

        # Initial DVH object and data
        self.save_data = {}
        self.group_data = {
            1: {
                "dvh": None,
                "data": {key: None for key in ["Plans", "Beams", "Rxs"]},
                "stats_data": None,
            },
            2: {
                "dvh": None,
                "data": {key: None for key in ["Plans", "Beams", "Rxs"]},
                "stats_data": None,
            },
        }

        self.toolbar_keys = [
            "Open",
            "Close",
            "Save",
            "Export",
            "Image",
            "Import",
            "Database",
            "ROI Map",
            "AI",
            "Settings",
        ]
        self.toolbar_ids = {
            key: i + 1000 for i, key in enumerate(self.toolbar_keys)
        }

        # sql_columns.py contains dictionaries of all queryable variables along with their
        # SQL columns and tables. Numerical categories include their units as well.
        self.categorical_columns = sql_columns.categorical
        self.numerical_columns = sql_columns.numerical

        # Keep track of currently selected row in the query tables
        self.selected_index_categorical = None
        self.selected_index_numerical = None

        # Load ROI Map now and pass to other objects for continuity
        # TODO: Need a method to address multiple users editing roi_map at the same time
        self.roi_map = DatabaseROIs()

        self.query_filters = None
        self.reset_query_filters()

        self.__add_menubar()
        self.__add_tool_bar()
        self.__add_layout_objects()
        self.__bind_layout_objects()

        columns = {
            "categorical": ["category_1", "category_2", "Filter Type"],
            "numerical": ["category", "min", "max", "Filter Type"],
        }
        self.data_table_categorical = DataTable(
            self.table_categorical, columns=columns["categorical"]
        )
        self.data_table_numerical = DataTable(
            self.table_numerical, columns=columns["numerical"]
        )

        self.__set_properties()
        self.__set_tooltips()
        self.__add_notebook_frames()
        self.__do_layout()

        self.disable_query_buttons("categorical")
        self.disable_query_buttons("numerical")
        self.button_query_execute.Disable()
        self.__disable_notebook_tabs()

        self.Bind(wx.EVT_CLOSE, self.on_quit)
        self.tool_bar_windows = {
            key: None for key in ["import", "database", "roi_map"]
        }
        self.loaded_mvr_ml_frames = []
        self.user_settings = None
        self.export_figure = None

        wx.CallAfter(self.__catch_failed_sql_connection_on_app_launch)

        self.correlation_error_displayed = False

        self.__do_subscribe()

        # Temporary dicompyler-core check until released on PyPI
        from dicompylercore import __version__ as dicompylercore_version

        if dicompylercore_version != "0.5.6":
            msg = (
                "dicompyler-core >= 0.5.6 is required for non-HFS imports. Please upgrade dicompyler-core with\n\n"
                "pip install --upgrade git+https://github.com/dicompyler/dicompyler-core.git\n\nand relaunch DVHA"
            )
            caption = "dicompyler-core Version Warning"
            ErrorDialog(self, msg, caption)

        if self.options.SHOW_NEW_PTV_CALC_WARNING:
            msg = (
                "As of DVHA v0.8.9, there are new PTV Distance calculations. "
                "Go to Data -> Database Administrator. Click Calculate for PTV Distances to ensure "
                "all of your data is calculated consistently across studies/patients.\n\n"
                "This warning will only show once. Must perform for each of your databases separately."
            )
            caption = "New PTV Distance Calculations"
            ErrorDialog(self, msg, caption)
            self.options.set_option("SHOW_NEW_PTV_CALC_WARNING", False)

        self.ml_views = []

    def __add_tool_bar(self):
        self.frame_toolbar = wx.ToolBar(
            self, -1, style=wx.TB_HORIZONTAL | wx.TB_TEXT
        )
        self.SetToolBar(self.frame_toolbar)

        description = {
            "Open": "Open previously queried data",
            "Close": "Clear queried data",
            "Save": "Save queried data",
            # 'Print': "Print a report",
            "Export": "Export data to CSV",
            "Image": "Export a plot to SVG/HTML",
            "Import": "DICOM import wizard",
            "Settings": "User Settings",
            "Database": "Database Administrator Tools",
            "ROI Map": "Define ROI name aliases",
            "AI": "Modeling with Scikit-Learn",
        }

        for key in self.toolbar_keys:
            bitmap = wx.Bitmap(ICONS[key], wx.BITMAP_TYPE_ANY)
            if is_windows() or is_linux():
                bitmap = scale_bitmap(bitmap, 30, 30)
            self.frame_toolbar.AddTool(
                self.toolbar_ids[key],
                key,
                bitmap,
                wx.NullBitmap,
                wx.ITEM_NORMAL,
                description[key],
                "",
            )

            if key in {"Close", "Image", "ROI Map", "AI"}:
                separator_count = (
                    4 if is_windows() else 1
                )  # MSW separator is very thin compared to mac
                for _ in range(separator_count):
                    self.frame_toolbar.AddSeparator()

        self.Bind(wx.EVT_TOOL, self.on_save, id=self.toolbar_ids["Save"])
        self.Bind(wx.EVT_TOOL, self.on_open, id=self.toolbar_ids["Open"])
        self.Bind(wx.EVT_TOOL, self.on_export, id=self.toolbar_ids["Export"])
        self.Bind(
            wx.EVT_TOOL, self.on_export_figure, id=self.toolbar_ids["Image"]
        )
        self.Bind(
            wx.EVT_TOOL,
            self.on_toolbar_database,
            id=self.toolbar_ids["Database"],
        )
        self.Bind(
            wx.EVT_TOOL,
            self.on_toolbar_settings,
            id=self.toolbar_ids["Settings"],
        )
        self.Bind(
            wx.EVT_TOOL,
            self.on_toolbar_roi_map,
            id=self.toolbar_ids["ROI Map"],
        )
        self.Bind(wx.EVT_TOOL, self.on_toolbar_ml, id=self.toolbar_ids["AI"])
        self.Bind(wx.EVT_TOOL, self.on_close, id=self.toolbar_ids["Close"])
        self.Bind(
            wx.EVT_TOOL, self.on_toolbar_import, id=self.toolbar_ids["Import"]
        )

    def __add_menubar(self):

        self.frame_menubar = wx.MenuBar()

        file_menu = wx.Menu()
        menu_open = file_menu.Append(wx.ID_OPEN, "&Open\tCtrl+O")
        menu_import = file_menu.Append(wx.ID_OPEN, "&Import DICOM\tCtrl+I")
        menu_save = file_menu.Append(wx.ID_ANY, "&Save\tCtrl+S")
        menu_close = file_menu.Append(wx.ID_ANY, "&Close")

        load_model = wx.Menu()
        load_model_mvr = load_model.Append(
            wx.ID_ANY, "Multi-Variable Regression"
        )
        load_model_ml = load_model.Append(wx.ID_ANY, "Machine Learning")

        export = wx.Menu()
        export_csv = export.Append(wx.ID_ANY, "Data to csv\tCtrl+E")
        export_figure = export.Append(wx.ID_ANY, "&Plot to file\tCtrl+P")
        export_pgsql = export.Append(wx.ID_ANY, "PGSQL to SQLite")
        export_db_to_json = export.Append(wx.ID_ANY, "Database to JSON")
        file_menu.AppendSeparator()

        qmi = file_menu.Append(wx.ID_ANY, "&Quit\tCtrl+Q")

        self.data_menu = wx.Menu()

        self.data_views = {
            key: None for key in ["DVHs", "Plans", "Rxs", "Beams"]
        }
        menu_db_admin = self.data_menu.Append(
            wx.ID_ANY, "Database Administrator"
        )
        self.data_menu.AppendSubMenu(load_model, "Load &Model")
        self.data_menu.AppendSubMenu(export, "&Export")
        menu_db_backup = self.data_menu.Append(wx.ID_ANY, "Backup SQLite DB")
        self.data_menu.AppendSeparator()
        self.data_menu_items = {
            "DVHs": self.data_menu.Append(wx.ID_ANY, "Show DVHs\tCtrl+1"),
            "Plans": self.data_menu.Append(wx.ID_ANY, "Show Plans\tCtrl+2"),
            "Rxs": self.data_menu.Append(wx.ID_ANY, "Show Rxs\tCtrl+3"),
            "Beams": self.data_menu.Append(wx.ID_ANY, "Show Beams\tCtrl+4"),
            "StatsData1": self.data_menu.Append(
                wx.ID_ANY, "Show Stats Data: Group 1\tCtrl+5"
            ),
            "StatsData2": self.data_menu.Append(
                wx.ID_ANY, "Show Stats Data: Group 2\tCtrl+6"
            ),
        }

        settings_menu = wx.Menu()
        menu_pref = settings_menu.Append(wx.ID_PREFERENCES)
        menu_sql = settings_menu.Append(
            wx.ID_ANY, "&Database Connection\tCtrl+D"
        )
        menu_sql_2 = settings_menu.Append(wx.ID_ANY, "&Database Connection 2")
        menu_roi_map = settings_menu.Append(wx.ID_ANY, "&ROI Map\tCtrl+R")

        help_menu = wx.Menu()
        menu_github = help_menu.Append(wx.ID_ANY, "GitHub Page")
        menu_dvha_edit = help_menu.Append(wx.ID_ANY, "DVHA DICOM Editor")
        menu_python_libraries = help_menu.Append(
            wx.ID_ANY, "Python Libraries Used"
        )
        menu_report_issue = help_menu.Append(wx.ID_ANY, "Report an Issue")
        menu_about = help_menu.Append(wx.ID_ANY, "&About")

        self.Bind(wx.EVT_MENU, self.on_quit, qmi)
        self.Bind(wx.EVT_MENU, self.on_open, menu_open)
        self.Bind(wx.EVT_MENU, self.on_toolbar_import, menu_import)
        self.Bind(wx.EVT_MENU, self.on_load_mvr_model, load_model_mvr)
        self.Bind(wx.EVT_MENU, self.on_load_ml_model, load_model_ml)
        self.Bind(wx.EVT_MENU, self.on_close, menu_close)
        self.Bind(wx.EVT_MENU, self.on_export, export_csv)
        self.Bind(wx.EVT_MENU, self.on_export_figure, export_figure)
        self.Bind(wx.EVT_MENU, self.on_export_pgsql, export_pgsql)
        self.Bind(wx.EVT_MENU, self.on_export_db_to_json, export_db_to_json)
        self.Bind(wx.EVT_MENU, self.on_save, menu_save)
        self.Bind(wx.EVT_MENU, self.on_pref, menu_pref)
        self.Bind(wx.EVT_MENU, self.on_githubpage, menu_github)
        self.Bind(wx.EVT_MENU, self.on_report_issue, menu_report_issue)
        self.Bind(wx.EVT_MENU, self.on_dvha_edit, menu_dvha_edit)
        self.Bind(wx.EVT_MENU, PythonLibraries, menu_python_libraries)
        self.Bind(wx.EVT_MENU, About, menu_about)
        self.Bind(wx.EVT_MENU, self.on_sql, menu_sql)
        self.Bind(wx.EVT_MENU, self.on_sql_2, menu_sql_2)
        self.Bind(wx.EVT_MENU, self.on_toolbar_roi_map, menu_roi_map)
        if is_mac():
            menu_user_settings = settings_menu.Append(
                wx.ID_ANY, "&Preferences\tCtrl+,"
            )
            self.Bind(wx.EVT_MENU, self.on_pref, menu_user_settings)
        menu_win_pos = settings_menu.Append(wx.ID_ANY, "Reset Windows")
        self.Bind(wx.EVT_MENU, self.on_reset_windows, menu_win_pos)

        self.Bind(wx.EVT_MENU, self.on_toolbar_database, menu_db_admin)
        self.Bind(wx.EVT_MENU, self.on_sqlite_backup, menu_db_backup)
        self.Bind(wx.EVT_MENU, self.on_view_dvhs, self.data_menu_items["DVHs"])
        self.Bind(
            wx.EVT_MENU, self.on_view_plans, self.data_menu_items["Plans"]
        )
        self.Bind(wx.EVT_MENU, self.on_view_rxs, self.data_menu_items["Rxs"])
        self.Bind(
            wx.EVT_MENU, self.on_view_beams, self.data_menu_items["Beams"]
        )
        self.Bind(
            wx.EVT_MENU,
            self.on_view_stats_data_1,
            self.data_menu_items["StatsData1"],
        )
        self.Bind(
            wx.EVT_MENU,
            self.on_view_stats_data_2,
            self.data_menu_items["StatsData2"],
        )

        self.frame_menubar.Append(file_menu, "&File")
        self.frame_menubar.Append(self.data_menu, "&Data")
        self.frame_menubar.Append(settings_menu, "&Settings")
        self.frame_menubar.Append(help_menu, "&Help")
        self.SetMenuBar(self.frame_menubar)

    def __add_layout_objects(self):
        self.button_categorical = {
            "add": wx.Button(self, wx.ID_ANY, "Add Filter"),
            "del": wx.Button(self, wx.ID_ANY, "Delete Selected"),
            "edit": wx.Button(self, wx.ID_ANY, "Edit Selected"),
        }
        self.button_numerical = {
            "add": wx.Button(self, wx.ID_ANY, "Add Filter"),
            "del": wx.Button(self, wx.ID_ANY, "Delete Selected"),
            "edit": wx.Button(self, wx.ID_ANY, "Edit Selected"),
        }

        self.table_categorical = wx.ListCtrl(
            self, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.LC_REPORT
        )
        self.table_numerical = wx.ListCtrl(
            self, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.LC_REPORT
        )

        self.radio_button_query_group = wx.RadioBox(
            self, wx.ID_ANY, "Query Group", choices=["1", "2"]
        )
        self.button_query_execute = wx.Button(
            self, wx.ID_ANY, "Query and Retrieve Group 1"
        )

        self.notebook_main_view = wx.Notebook(self, wx.ID_ANY)
        self.tab_keys = [
            "Welcome",
            "DVHs",
            "Endpoints",
            "Rad Bio",
            "Time Series",
            "Correlation",
            "Regression",
            "Control Chart",
        ]
        self.notebook_tab = {
            key: wx.Panel(self.notebook_main_view, wx.ID_ANY)
            for key in self.tab_keys
        }

        # self.text_summary = wx.StaticText(
        #     self, wx.ID_ANY, "", style=wx.ALIGN_LEFT
        # )
        self.text_summary = wx.TextCtrl(
            self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY
        )

    def __bind_layout_objects(self):
        self.Bind(
            wx.EVT_LIST_ITEM_SELECTED,
            self.on_item_select_categorical,
            self.table_categorical,
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.add_row_categorical,
            id=self.button_categorical["add"].GetId(),
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.del_row_categorical,
            id=self.button_categorical["del"].GetId(),
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.edit_row_categorical,
            id=self.button_categorical["edit"].GetId(),
        )
        self.Bind(
            wx.EVT_LIST_ITEM_ACTIVATED,
            self.doubleclick_categorical,
            self.table_categorical,
        )

        self.Bind(
            wx.EVT_LIST_ITEM_SELECTED,
            self.on_item_select_numerical,
            self.table_numerical,
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.add_row_numerical,
            id=self.button_numerical["add"].GetId(),
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.del_row_numerical,
            id=self.button_numerical["del"].GetId(),
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.edit_row_numerical,
            id=self.button_numerical["edit"].GetId(),
        )
        self.Bind(
            wx.EVT_LIST_ITEM_ACTIVATED,
            self.doubleclick_numerical,
            self.table_numerical,
        )

        self.Bind(
            wx.EVT_RADIOBOX,
            self.on_group_select,
            id=self.radio_button_query_group.GetId(),
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.exec_query_button,
            id=self.button_query_execute.GetId(),
        )

        self.Bind(wx.EVT_SIZE, self.on_resize)
        self.Bind(wx.EVT_MOVE, self.on_move)

    def __set_properties(self):
        self.SetTitle("DVH Analytics")

        self.frame_toolbar.Realize()

        widths = [180, 150, 80]
        self.table_categorical.AppendColumn(
            "Category1", format=wx.LIST_FORMAT_LEFT, width=widths[0]
        )
        self.table_categorical.AppendColumn(
            "Category2", format=wx.LIST_FORMAT_LEFT, width=widths[1]
        )
        self.table_categorical.AppendColumn(
            "Filter Type", format=wx.LIST_FORMAT_LEFT, width=widths[2]
        )
        self.data_table_categorical.widths = widths

        widths = [150, 90, 90, 80]
        self.table_numerical.AppendColumn(
            "Category", format=wx.LIST_FORMAT_LEFT, width=widths[0]
        )
        self.table_numerical.AppendColumn(
            "Min", format=wx.LIST_FORMAT_LEFT, width=widths[1]
        )
        self.table_numerical.AppendColumn(
            "Max", format=wx.LIST_FORMAT_LEFT, width=widths[2]
        )
        self.table_numerical.AppendColumn(
            "Filter Type", format=wx.LIST_FORMAT_LEFT, width=widths[3]
        )
        self.data_table_numerical.widths = widths

    def __set_tooltips(self):
        self.button_categorical["add"].SetToolTip(
            "Add a categorical data filter."
        )
        self.button_categorical["del"].SetToolTip(
            "Delete the currently selected category filter."
        )
        self.button_categorical["edit"].SetToolTip(
            "Edit the currently selected category filter."
        )
        self.button_numerical["add"].SetToolTip("Add a numerical data filter.")
        self.button_numerical["del"].SetToolTip(
            "Delete the currently selected numerical data filter."
        )
        self.button_numerical["edit"].SetToolTip(
            "Edit the currently selected data filter."
        )
        self.button_query_execute.SetToolTip(
            "Query the database with the filters entered above. At least one "
            "filter must be added."
        )

    def __add_notebook_frames(self):
        self.plot = PlotStatDVH(
            self.notebook_tab["DVHs"], self.group_data, self.options
        )
        self.time_series = TimeSeriesFrame(self)
        self.correlation = CorrelationFrame(self)
        self.regression = RegressionFrame(self)
        self.control_chart = ControlChartFrame(self)
        self.radbio = RadBioFrame(self)
        self.endpoint = EndpointFrame(self)

    def __do_layout(self):
        sizer_summary = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, "Summary"), wx.HORIZONTAL
        )
        sizer_query_numerical = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, "Query by Numerical Data"),
            wx.VERTICAL,
        )

        sizer_query_categorical = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, "Query by Categorical Data"),
            wx.VERTICAL,
        )

        sizer_categorical_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_numerical_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_query_exec_buttons = wx.BoxSizer(wx.HORIZONTAL)

        for key in ["add", "del", "edit"]:
            sizer_categorical_buttons.Add(
                self.button_categorical[key], 0, wx.ALL, 5
            )
            sizer_numerical_buttons.Add(
                self.button_numerical[key], 0, wx.ALL, 5
            )

        sizer_query_categorical.Add(
            sizer_categorical_buttons, 0, wx.ALL | wx.EXPAND, 5
        )
        sizer_query_categorical.Add(
            self.table_categorical, 1, wx.ALL | wx.EXPAND, 10
        )

        sizer_query_numerical.Add(
            sizer_numerical_buttons, 0, wx.ALL | wx.EXPAND, 5
        )
        sizer_query_numerical.Add(
            self.table_numerical, 1, wx.ALL | wx.EXPAND, 10
        )

        sizer_summary.Add(self.text_summary, 1, wx.EXPAND | wx.ALL, 5)

        panel_left = wx.BoxSizer(wx.VERTICAL)
        panel_left.Add(
            sizer_query_categorical,
            0,
            wx.EXPAND | wx.LEFT | wx.RIGHT | wx.SHAPED | wx.TOP,
            5,
        )
        panel_left.Add(
            sizer_query_numerical,
            0,
            wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT | wx.SHAPED,
            5,
        )
        sizer_query_exec_buttons.Add(self.radio_button_query_group, 0, 0, 0)
        sizer_query_exec_buttons.Add(
            self.button_query_execute, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 10
        )
        panel_left.Add(
            sizer_query_exec_buttons, 0, wx.EXPAND | wx.RIGHT | wx.LEFT, 5
        )
        panel_left.Add(sizer_summary, 1, wx.ALL | wx.EXPAND, 5)

        bitmap_logo = wx.StaticBitmap(
            self.notebook_tab["Welcome"],
            wx.ID_ANY,
            wx.Bitmap(LOGO_PATH, wx.BITMAP_TYPE_ANY),
        )
        text_welcome = wx.StaticText(
            self.notebook_tab["Welcome"],
            wx.ID_ANY,
            "\n\nWelcome to DVH Analytics.\nIf you already have a database built, design "
            "a query with the filters to the left.\n\n\n\nDVH Analytics is a software "
            "application to help radiation oncology departments build an in-house database "
            "of treatment planning data for the purpose of historical comparisons and "
            "statistical analysis. This code is still in development. Please contact the "
            "developer if you are interested in testing or collaborating.\n\nThe application "
            "builds a SQL database of DVHs and various planning parameters from DICOM files "
            "(i.e., Plan, Structure, Dose). Since the data is extracted directly from DICOM "
            "files, we intend to accommodate an array of treatment planning system vendors.",
            style=wx.ALIGN_CENTER,
        )

        text_welcome_size = get_window_size(0.417, 0.476)
        text_welcome.SetMinSize(text_welcome_size)
        text_welcome.Wrap(text_welcome_size[1])

        sizer_welcome = wx.BoxSizer(wx.VERTICAL)
        sizer_welcome.Add(
            bitmap_logo, 0, wx.ALIGN_CENTER | wx.LEFT | wx.RIGHT | wx.TOP, 100
        )
        sizer_welcome.Add(text_welcome, 0, wx.ALIGN_CENTER | wx.ALL, 25)
        self.notebook_tab["Welcome"].SetSizer(sizer_welcome)

        self.sizer_dvhs.Add(self.plot.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab["DVHs"].SetSizer(self.sizer_dvhs)

        sizer_endpoint = wx.BoxSizer(wx.VERTICAL)
        sizer_endpoint.Add(self.endpoint.layout, 0, wx.ALL, 25)
        self.notebook_tab["Endpoints"].SetSizer(sizer_endpoint)

        sizer_rad_bio = wx.BoxSizer(wx.VERTICAL)
        sizer_rad_bio.Add(self.radbio.layout, 0, wx.ALL, 25)
        self.notebook_tab["Rad Bio"].SetSizer(sizer_rad_bio)

        sizer_time_series = wx.BoxSizer(wx.VERTICAL)
        sizer_time_series.Add(
            self.time_series.layout, 1, wx.EXPAND | wx.ALL, 25
        )
        self.notebook_tab["Time Series"].SetSizer(sizer_time_series)

        sizer_correlation = wx.BoxSizer(wx.VERTICAL)
        sizer_correlation.Add(
            self.correlation.layout, 1, wx.EXPAND | wx.ALL, 25
        )
        self.notebook_tab["Correlation"].SetSizer(sizer_correlation)

        sizer_regression = wx.BoxSizer(wx.VERTICAL)
        sizer_regression.Add(self.regression.layout, 1, wx.EXPAND | wx.ALL, 25)
        self.notebook_tab["Regression"].SetSizer(sizer_regression)

        sizer_control_chart = wx.BoxSizer(wx.VERTICAL)
        sizer_control_chart.Add(
            self.control_chart.layout, 1, wx.EXPAND | wx.ALL, 25
        )
        self.notebook_tab["Control Chart"].SetSizer(sizer_control_chart)

        for key in self.tab_keys:
            self.notebook_main_view.AddPage(self.notebook_tab[key], key)

        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_main_wrapper = wx.BoxSizer(wx.HORIZONTAL)
        hbox_main = wx.BoxSizer(wx.HORIZONTAL)
        hbox_main.Add(
            panel_left, 0, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.TOP, 5
        )
        hbox_main.Add(
            self.notebook_main_view,
            1,
            wx.BOTTOM | wx.EXPAND | wx.RIGHT | wx.TOP,
            5,
        )
        sizer_main_wrapper.Add(hbox_main, 1, wx.EXPAND, 0)
        sizer_main.Add(sizer_main_wrapper, 1, wx.EXPAND, 0)

        self.SetSizer(sizer_main)
        self.Layout()

        self.__apply_size_and_position()

    def __apply_size_and_position(self):
        if self.options.window_sizes["main"] is not None:
            size = self.options.window_sizes["main"]
        else:
            size = get_window_size(0.833, 0.875)
            size = apply_resolution_limits(size, self.options)

        self.SetSize(size)
        self.options.apply_window_position(self, "main")

        self.allow_window_size_save = True

    def __enable_notebook_tabs(self):
        for key in self.tab_keys:
            self.notebook_tab[key].Enable()
        self.__enable_initial_buttons_in_tabs()

    def __disable_notebook_tabs(self):
        for key in self.tab_keys:
            if key != "Welcome":
                self.notebook_tab[key].Disable()

    def __enable_initial_buttons_in_tabs(self):
        self.endpoint.enable_initial_buttons()
        self.radbio.enable_initial_buttons()
        self.time_series.enable_initial_buttons()

    def __disable_add_filter_buttons(self):
        self.button_categorical["add"].Disable()
        self.button_numerical["add"].Disable()

    def __enable_add_filter_buttons(self):
        self.button_categorical["add"].Enable()
        self.button_numerical["add"].Enable()

    def enable_query_buttons(self, query_type):
        for key in ["del", "edit"]:
            {
                "categorical": self.button_categorical[key],
                "numerical": self.button_numerical[key],
            }[query_type].Enable()

    def disable_query_buttons(self, query_type):
        for key in ["del", "edit"]:
            {
                "categorical": self.button_categorical[key],
                "numerical": self.button_numerical[key],
            }[query_type].Disable()

    def update_all_query_buttons(self):
        tables = {
            "numerical": self.data_table_numerical,
            "categorical": self.data_table_categorical,
        }
        for key, table in tables.items():
            if table.data is not None:
                [self.disable_query_buttons, self.enable_query_buttons][
                    table.row_count > 0
                ](key)
            else:
                self.disable_query_buttons(key)

        if (
            self.data_table_numerical.row_count
            + self.data_table_categorical.row_count
            > 0
        ):
            self.button_query_execute.Enable()
        else:
            self.button_query_execute.Disable()

        # Force user to populate group 1 first
        if self.selected_group == 2 and self.group_data[1]["dvh"] is None:
            self.button_query_execute.Disable()

    def __catch_failed_sql_connection_on_app_launch(self):
        if self.options.DB_TYPE_GRPS[1] == "pgsql":
            if not echo_sql_db():
                wx.MessageBox(
                    "Invalid credentials!",
                    "Echo SQL Database",
                    wx.OK | wx.ICON_WARNING,
                )
                self.on_sql()
        else:  # if using sqlite
            initialize_db()

    def __do_subscribe(self):
        pub.subscribe(self.raise_error_dialog, "import_status_raise_error")
        pub.subscribe(self.call_sqlite_backup, "backup_sqlite_db")
        pub.subscribe(
            self.correlation_patients_removed, "correlation_patients_removed"
        )

    def raise_error_dialog(self, msg):
        MemoryErrorDialog(self, msg)

    # --------------------------------------------------------------------------------------------------------------
    # Menu bar event functions
    # --------------------------------------------------------------------------------------------------------------
    def on_save(self, evt):
        if self.save_data:
            dlg = wx.FileDialog(
                self,
                "Save your session data to file",
                "",
                wildcard="*.dvha",
                style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
            )
            dlg.SetDirectory(DATA_DIR)
            if dlg.ShowModal() == wx.ID_OK:
                self.save_data_obj()
                save_object_to_file(self.save_data, dlg.GetPath())
            dlg.Destroy()
        else:
            wx.MessageBox(
                "There is no data to save. Please query/open some data first.",
                "Save Error",
                wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING,
            )

    def on_open(self, evt):
        dlg = wx.FileDialog(
            self,
            "Open saved data",
            "",
            wildcard="*.dvha",
            style=wx.FD_FILE_MUST_EXIST | wx.FD_OPEN,
        )
        if dlg.ShowModal() == wx.ID_OK:
            self.close()
            self.load_data_obj(dlg.GetPath())

        dlg.Destroy()

    def on_load_mvr_model(self, *evt):
        with wx.FileDialog(
            self,
            "Load a multi-variable linear regression model",
            "",
            wildcard="*.mvr",
            style=wx.FD_FILE_MUST_EXIST | wx.FD_OPEN,
        ) as dlg:
            dlg.SetDirectory(MODELS_DIR)
            if dlg.ShowModal() == wx.ID_OK:
                model_file_path = dlg.GetPath()
                dlg.Destroy()
                self.loaded_mvr_ml_frames.append(
                    LoadMultiVarModelFrame(
                        self,
                        model_file_path,
                        self.group_data,
                        self.selected_group,
                        self.options,
                    )
                )

    def on_load_ml_model(self, *evt):
        if self.group_data[self.selected_group]["stats_data"]:
            self.loaded_mvr_ml_frames.append(
                MachineLearningModelViewer(
                    self, self.group_data, self.selected_group, self.options
                )
            )
        else:
            wx.MessageBox(
                "No data as been queried for Group %s." % self.selected_group,
                "Error",
                wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING,
            )

    @property
    def selected_group(self):
        return self.radio_button_query_group.GetSelection() + 1

    def save_data_obj(self):
        self.save_data["group_data"] = self.group_data
        self.save_data["query_filters"] = self.query_filters
        self.save_data["time_stamp"] = datetime.now()
        self.save_data["version"] = DefaultOptions().VERSION
        # data_table_categorical and data_table_numerical saved after query to ensure these data reflect
        # the rest of the saved data
        self.save_data["endpoint"] = self.endpoint.get_save_data()
        self.save_data["time_series"] = self.time_series.get_save_data()
        self.save_data["radbio"] = self.radbio.get_save_data()
        self.save_data["regression"] = self.regression.get_save_data()
        self.save_data["control_chart"] = self.control_chart.get_save_data()

    def load_data_obj(self, abs_file_path):
        self.save_data = load_object_from_file(abs_file_path)
        self.group_data = self.save_data["group_data"]

        # .load_save_data loses column widths?
        self.radio_button_query_group.SetSelection(0)
        self.data_table_categorical.load_save_data(
            self.save_data["main_categorical_1"]
        )
        self.data_table_numerical.load_save_data(
            self.save_data["main_numerical_1"]
        )

        self.control_chart.load_save_data(self.save_data["control_chart"])

        self.radio_button_query_group.SetSelection(0)
        self.exec_query(load_saved_dvh_data=True, group=1)

        if "main_categorical_2" in self.save_data.keys():
            self.radio_button_query_group.SetSelection(1)
            self.on_group_select()
            self.data_table_categorical.load_save_data(
                self.save_data["main_categorical_2"]
            )
            self.data_table_numerical.load_save_data(
                self.save_data["main_numerical_2"]
            )
            self.update_all_query_buttons()

            self.exec_query(load_saved_dvh_data=True, group=2)

        self.update_all_query_buttons()

        self.endpoint.load_save_data(self.save_data["endpoint"])
        if self.endpoint.has_data:
            self.endpoint.enable_buttons()

        self.radbio.load_save_data(self.save_data["radbio"])

        self.endpoint.update_endpoints_in_dvh()

        self.group_data[1]["stats_data"].update_endpoints_and_radbio()
        if self.group_data[2]["stats_data"]:
            self.group_data[2]["stats_data"].update_endpoints_and_radbio()

        self.time_series.load_save_data(self.save_data["time_series"])
        self.time_series.update_plot()

        self.regression.update_combo_box_choices()
        self.regression.load_save_data(self.save_data["regression"])

        self.control_chart.update_combo_box_y_choices()

    def on_toolbar_settings(self, evt):
        self.on_pref()

    def on_toolbar_import(self, evt):
        self.check_db_then_call(
            ImportDicomFrame, "import", self.roi_map, self.options
        )

    def on_toolbar_database(self, evt):
        self.check_db_then_call(
            DatabaseEditorFrame, "database", self.roi_map, self.options
        )

    def on_toolbar_roi_map(self, evt):
        self.check_db_then_call(ROIMapFrame, "roi_map", self.roi_map)

    def on_toolbar_ml(self, evt):
        if self.group_data[self.selected_group]["dvh"] is not None:
            dlg = MachineLearningSetupDlg(
                self.group_data[self.selected_group]["stats_data"],
                self.options,
            )
            res = dlg.ShowModal()
            if res == wx.ID_OK:
                frame = ALGORITHMS[dlg.ml_alg]["frame"]
                self.ml_views.append(
                    frame(self, dlg.ml_input_data, alg_type=dlg.alg_type)
                )
            dlg.Destroy()
        else:
            wx.MessageBox(
                "There is no data to use! Please query some data first.",
                "Data Error",
                wx.OK | wx.ICON_WARNING,
            )

    def on_sqlite_backup(self, *evt):
        wx.CallAfter(do_sqlite_backup, self, self.options)

    def call_sqlite_backup(self):
        if self.options.AUTO_SQL_DB_BACKUP:
            self.on_sqlite_backup()

    def check_db_then_call(self, func, window_type, *parameters):
        if not echo_sql_db():
            self.on_sql()

        if echo_sql_db():
            if self.tool_bar_windows[window_type]:
                self.tool_bar_windows[window_type].Raise()
            else:
                self.tool_bar_windows[window_type] = func(*parameters)
        else:
            wx.MessageBox(
                "Connection to SQL database could not be established.",
                "Connection Error",
                wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING,
            )

    def on_export_pgsql(self, evt):
        if self.options.DB_TYPE_GRPS[1] == "pgsql":
            dlg = wx.FileDialog(
                self,
                "Export your PGSQL DB to SQLite",
                "",
                wildcard="*.db",
                style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
            )
            dlg.SetDirectory(DATA_DIR)
            if dlg.ShowModal() == wx.ID_OK:
                ExportPGSQLProgressFrame(dlg.GetPath())
        else:
            wx.MessageBox(
                "This is for PostgreSQL only. Your main (Group 1) connection is SQLite. "
                "Use Data->Backup SQLite DB instead.",
                "DB Export Error",
                wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING,
            )

    def on_export_db_to_json(self, evt):
        dlg = wx.FileDialog(
            self,
            "Export your entire DB to JSON",
            "",
            wildcard="*.json",
            style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
        )
        dlg.SetDirectory(DATA_DIR)
        if dlg.ShowModal() == wx.ID_OK:
            ExportPGSQLProgressFrame(dlg.GetPath(), export_to_json=True)

    # --------------------------------------------------------------------------------------------------------------
    # Query event functions
    # --------------------------------------------------------------------------------------------------------------
    def on_item_select_categorical(self, evt):
        self.selected_index_categorical = (
            self.table_categorical.GetFirstSelected()
        )

    def on_item_select_numerical(self, evt):
        self.selected_index_numerical = self.table_numerical.GetFirstSelected()

    def add_row_categorical(self, evt):
        query_dlg(self, "categorical")
        self.button_query_execute.Enable()
        self.update_all_query_buttons()

    def add_row_numerical(self, evt):
        query_dlg(self, "numerical")
        self.update_all_query_buttons()

    def edit_row_categorical(self, *args):
        if self.selected_index_categorical is not None:
            query_dlg(self, "categorical", set_values=True)

    def edit_row_numerical(self, *args):
        if self.selected_index_numerical is not None:
            query_dlg(self, "numerical", set_values=True)

    def doubleclick_categorical(self, evt):
        self.selected_index_categorical = (
            self.table_categorical.GetFirstSelected()
        )
        self.edit_row_categorical()

    def doubleclick_numerical(self, evt):
        self.selected_index_numerical = self.table_numerical.GetFirstSelected()
        self.edit_row_numerical()

    def del_row_categorical(self, evt):
        self.data_table_categorical.delete_row(self.selected_index_categorical)
        self.selected_index_categorical = None
        self.update_all_query_buttons()

    def del_row_numerical(self, evt):
        self.data_table_numerical.delete_row(self.selected_index_numerical)
        self.selected_index_numerical = None
        self.update_all_query_buttons()

    def exec_query_button(self, evt):
        self.exec_query()

    def exec_query(self, load_saved_dvh_data=False, group=None):

        # Used to make sure Correlation calculation error of patient removal is only displayed once after a query
        self.correlation_error_displayed = False

        wx.BeginBusyCursor()
        if group is not None:
            self.radio_button_query_group.SetSelection(group - 1)
        group = self.selected_group

        # TODO: retain group 1 endpoint defs after query of group 2
        self.endpoint.clear_data()

        if group == 1:
            self.plot.clear_plot()
            self.time_series.clear_data()
            self.regression.clear(self.group_data)
            self.control_chart.clear_data()
            self.radbio.clear_data()

        if not load_saved_dvh_data:
            try:
                uids, dvh_str = self.get_query()
                self.group_data[group]["dvh"] = DVH(
                    dvh_condition=dvh_str,
                    uid=uids,
                    dvh_bin_width=self.options.dvh_bin_width,
                    group=group,
                )
            except MemoryError:
                msg = (
                    "Querying memory error. Try querying less data. At least %s DVHs returned.\n"
                    "NOTE: Threshold of this error is dependent on your computer."
                    % self.group_data[group]["dvh"].count
                )
                MemoryErrorDialog(self, msg)
                self.close()
                return

        count = self.group_data[group]["dvh"].count
        min_dvh_count = 3
        if count >= min_dvh_count:
            try:
                self.endpoint.update_dvh(self.group_data)
                self.set_summary_text(group)
                self.plot.update_plot(
                    self.group_data[1]["dvh"], dvh_2=self.group_data[2]["dvh"]
                )
                wx.EndBusyCursor()
                self.update_data(
                    load_saved_dvh_data=load_saved_dvh_data,
                    group_2_only=bool(group - 1),
                )

                if group == 1:
                    self.notebook_main_view.SetSelection(1)
                    self.__enable_notebook_tabs()

                self.save_data[
                    "main_categorical_%s" % group
                ] = self.data_table_categorical.get_save_data()
                self.save_data[
                    "main_numerical_%s" % group
                ] = self.data_table_numerical.get_save_data()

                if group == 2:
                    self.regression.group = 2
                    self.control_chart.group = 2
                    self.update_stats_data_plots()

            except PlottingMemoryError as e:
                wx.EndBusyCursor()
                self.on_plotting_memory_error(str(e))
        else:
            wx.EndBusyCursor()
            msg = (
                "%s DVHs returned. Please modify query or import more data."
                % ["Less than %s" % min_dvh_count, "No"][count == 0]
            )
            wx.MessageBox(
                msg, "Query Error", wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING
            )
            self.group_data[group]["dvh"] = None

    def get_query(self):

        # Used to accumulate lists of query strings for each table
        # Will assume each item in list is complete query for that SQL column
        queries = {"Plans": [], "Rxs": [], "Beams": [], "DVHs": []}

        # Used to group queries by variable, will combine all queries of same variable with an OR operator
        # e.g., queries_by_sql_column['Plans'][key] = list of strings, where key is sql column
        queries_by_sql_column = {
            "Plans": {},
            "Rxs": {},
            "Beams": {},
            "DVHs": {},
        }
        operator_by_sql_column = {
            "Plans": {},
            "Rxs": {},
            "Beams": {},
            "DVHs": {},
        }

        # Categorical filter
        if self.data_table_categorical.row_count:
            for i, category in enumerate(
                self.data_table_categorical.data["category_1"]
            ):
                table = self.categorical_columns[category]["table"]
                col = self.categorical_columns[category]["var_name"]
                value = self.data_table_categorical.data["category_2"][i]
                if col not in queries_by_sql_column[table]:
                    queries_by_sql_column[table][col] = []
                    operator_by_sql_column[table][col] = []
                operator = ["=", "!="][
                    {"Include": 0, "Exclude": 1}[
                        self.data_table_categorical.data["Filter Type"][i]
                    ]
                ]
                queries_by_sql_column[table][col].append(
                    "%s %s '%s'" % (col, operator, value)
                )
                operator_by_sql_column[table][col].append(
                    self.data_table_categorical.data["Filter Type"][i]
                )

        # Range filter
        if self.data_table_numerical.row_count:
            for i, category in enumerate(
                self.data_table_numerical.data["category"]
            ):
                table = self.numerical_columns[category]["table"]
                col = self.numerical_columns[category]["var_name"]
                value_low = self.data_table_numerical.data["min"][i]
                value_high = self.data_table_numerical.data["max"][i]
                if "date" in col:
                    value_low = "'%s'" % value_low
                    value_high = "'%s'" % value_high
                    if self.options.DB_TYPE_GRPS[1] == "pgsql":
                        value_low = value_low + "::date"
                        value_high = value_high + "::date"
                if col not in queries_by_sql_column[table]:
                    queries_by_sql_column[table][col] = []
                    operator_by_sql_column[table][col] = []
                operator = ["BETWEEN", "NOT BETWEEN"][
                    {"Include": 0, "Exclude": 1}[
                        self.data_table_numerical.data["Filter Type"][i]
                    ]
                ]
                queries_by_sql_column[table][col].append(
                    "%s %s %s AND %s" % (col, operator, value_low, value_high)
                )
                operator_by_sql_column[table][col].append(
                    self.data_table_numerical.data["Filter Type"][i]
                )

        for table in queries:
            for col in queries_by_sql_column[table]:
                includes = [
                    item
                    for i, item in enumerate(queries_by_sql_column[table][col])
                    if operator_by_sql_column[table][col][i]
                ]
                excludes = [
                    item
                    for i, item in enumerate(queries_by_sql_column[table][col])
                    if not operator_by_sql_column[table][col][i]
                ]

                includes_str = (
                    "(%s)" % " OR ".join(includes) if includes else None
                )
                excludes_str = (
                    "(%s)" % " AND ".join(excludes) if excludes else None
                )

                queries[table].append(
                    "(%s)"
                    % " AND ".join(
                        [item for item in [includes_str, excludes_str] if item]
                    )
                )
            queries[table] = " AND ".join(queries[table])

        uids = get_study_instance_uids(
            plans=queries["Plans"], rxs=queries["Rxs"], beams=queries["Beams"]
        )["common"]

        return uids, queries["DVHs"]

    def update_data(self, load_saved_dvh_data=False, group_2_only=False):
        wx.BeginBusyCursor()
        tables = ["Plans", "Rxs", "Beams"]
        for grp, grp_data in self.group_data.items():
            if not (grp == 1 and group_2_only) or grp == 2:
                if hasattr(grp_data["dvh"], "study_instance_uid"):
                    if not load_saved_dvh_data:
                        condition_str = (
                            "study_instance_uid in ('%s')"
                            % "','".join(grp_data["dvh"].study_instance_uid)
                        )
                        grp_data["data"] = {
                            key: QuerySQL(key, condition_str) for key in tables
                        }
                        grp_data["stats_data"] = StatsData(
                            grp_data["dvh"], grp_data["data"], group=grp
                        )
                else:
                    grp_data["data"] = {key: None for key in tables}
                    grp_data["stats_data"] = None

        if self.group_data[2]["stats_data"]:
            sync_variables_in_stats_data_objects(
                self.group_data[1]["stats_data"],
                self.group_data[2]["stats_data"],
            )
        self.time_series.update_data(self.group_data)
        self.control_chart.update_data(self.group_data)
        self.correlation.set_data(self.group_data)
        self.regression.update_combo_box_choices()
        self.radbio.update_dvh_data(self.group_data)

        wx.EndBusyCursor()

    # --------------------------------------------------------------------------------------------------------------
    # Menu bar event functions
    # --------------------------------------------------------------------------------------------------------------
    def close_windows(self):
        for view in self.data_views.values():
            if hasattr(view, "Destroy"):
                try:
                    view.Destroy()
                except RuntimeError:
                    pass

        if self.user_settings is not None:
            self.user_settings.Close()

        if self.export_figure is not None:
            self.export_figure.Close()

        for window in self.tool_bar_windows.values():
            if window and hasattr(window, "Destroy"):
                window.Destroy()

        for view in self.ml_views:
            if hasattr(view, "Destroy"):
                try:
                    view.Destroy()
                except RuntimeError:
                    pass

        self.regression.close_mvr_frames()

    def on_quit(self, evt):
        self.close_windows()
        self.Destroy()

    def on_close(self, *evt):
        if self.group_data[1]["dvh"]:
            dlg = wx.MessageDialog(
                self,
                "Clear all data and plots?",
                caption="Close",
                style=wx.YES
                | wx.NO
                | wx.NO_DEFAULT
                | wx.CENTER
                | wx.ICON_EXCLAMATION,
            )
            dlg.Center()
            res = dlg.ShowModal()
            if res == wx.ID_YES:
                self.close()
                self.radio_button_query_group.SetSelection(0)
            dlg.Destroy()

    def close(self):
        self.group_data = {
            1: {
                "dvh": None,
                "data": {key: None for key in ["Plans", "Beams", "Rxs"]},
                "stats_data": None,
            },
            2: {
                "dvh": None,
                "data": {key: None for key in ["Plans", "Beams", "Rxs"]},
                "stats_data": None,
            },
        }
        self.data_table_categorical.delete_all_rows()
        self.data_table_numerical.delete_all_rows()
        self.plot.clear_plot()
        self.endpoint.clear_data()
        self.radbio.clear_data()
        self.time_series.clear_data()
        self.notebook_main_view.SetSelection(0)
        self.text_summary.SetValue("")
        self.__disable_notebook_tabs()
        self.disable_query_buttons("categorical")
        self.disable_query_buttons("numerical")
        self.button_query_execute.Disable()
        self.time_series.initialize_y_axis_options()
        self.regression.clear(self.group_data)
        self.control_chart.initialize_y_axis_options()
        self.control_chart.plot.clear_plot()
        self.control_chart.group = 1
        self.close_windows()
        self.reset_query_filters()

    def on_export(self, evt):
        if self.group_data[1]["dvh"] is not None:
            ExportCSVDialog(self)
        else:
            wx.MessageBox(
                "There is no data to export! Please query some data first.",
                "Export Error",
                wx.OK | wx.ICON_WARNING,
            )

    @staticmethod
    def on_githubpage(evt):
        webbrowser.open_new_tab("http://dvhanalytics.com/")

    @staticmethod
    def on_dvha_edit(evt):
        webbrowser.open_new_tab(
            "https://github.com/cutright/DVHA-DICOM-Editor"
        )

    @staticmethod
    def on_report_issue(evt):
        webbrowser.open_new_tab(
            "https://github.com/cutright/DVH-Analytics/issues"
        )

    def on_pref(self, *args):
        if self.user_settings is None:
            self.user_settings = UserSettings(self)
            self.user_settings.Show()
        else:
            self.user_settings.Raise()

    def on_sql(self, *args):
        self.open_sql_setting(group=1)

    def on_sql_2(self, *args):
        self.open_sql_setting(group=2)

    def open_sql_setting(self, group):
        SQLSettingsDialog(self.options, group)
        [self.__disable_add_filter_buttons, self.__enable_add_filter_buttons][
            echo_sql_db(group=self.selected_group)
        ]()

    def on_export_figure(self, evt):
        if self.save_data:
            if self.export_figure is None:
                try:
                    self.export_figure = ExportFigure(self)
                except Exception:
                    self.options.save_fig_param = (
                        DefaultOptions().save_fig_param
                    )
                    self.export_figure = ExportFigure(self)
                self.export_figure.Show()
            else:
                self.export_figure.Raise()
        else:
            wx.MessageBox(
                "There is no data to save. Please query/open some data first.",
                "Save Error",
                wx.OK | wx.OK_DEFAULT | wx.ICON_WARNING,
            )

    def on_view_dvhs(self, evt):
        self.view_table_data("DVHs")

    def on_view_plans(self, evt):
        self.view_table_data("Plans")

    def on_view_rxs(self, evt):
        self.view_table_data("Rxs")

    def on_view_beams(self, evt):
        self.view_table_data("Beams")

    def on_view_stats_data_1(self, evt):
        self.view_table_data("StatsData1")

    def on_view_stats_data_2(self, evt):
        self.view_table_data("StatsData2")

    def view_table_data(self, key):
        if key == "DVHs":
            data = {grp: self.group_data[grp]["dvh"] for grp in [1, 2]}
        elif "StatsData" in key:
            data = {grp: self.group_data[grp]["stats_data"] for grp in [1, 2]}
        else:
            data = {grp: self.group_data[grp]["data"][key] for grp in [1, 2]}

        if data[1]:

            if self.get_menu_item_status(key) == "Show":
                if "StatsData" in key:
                    group = int(key[-1])
                    if group == 1 or data[2] is not None:
                        self.data_views[key] = StatsDataEditor(
                            self.group_data,
                            group,
                            self.data_menu,
                            self.data_menu_items[key].GetId(),
                            self.time_series,
                            self.regression,
                            self.control_chart,
                        )
                    else:
                        self.no_queried_data_dlg()
                else:
                    if key == "DVHs":
                        columns = [c for c in data[1].keys]
                    elif key == "Rxs":
                        columns = [
                            "plan_name",
                            "fx_dose",
                            "rx_percent",
                            "fxs",
                            "rx_dose",
                            "fx_grp_number",
                            "fx_grp_count",
                            "fx_grp_name",
                            "normalization_method",
                            "normalization_object",
                        ]
                    else:
                        columns = [
                            obj["var_name"]
                            for obj in sql_column_info.values()
                            if obj["table"] == key
                        ]

                    for starter_column in ["study_instance_uid", "mrn"]:
                        if starter_column in columns:
                            columns.pop(columns.index(starter_column))
                        columns.insert(0, starter_column)

                    self.data_views[key] = QueriedDataFrame(
                        data,
                        columns,
                        key,
                        self.data_menu,
                        self.data_menu_items[key].GetId(),
                    )
            else:
                self.data_views[key].on_close()
                self.data_views[key] = None
        else:
            self.no_queried_data_dlg()

    def no_queried_data_dlg(self):
        dlg = wx.MessageDialog(
            self,
            "Please query/open some data first.",
            "ERROR!",
            wx.ICON_ERROR | wx.OK_DEFAULT,
        )
        dlg.ShowModal()
        dlg.Destroy()

    def get_menu_item_status(self, key):
        show_hide = ["Hide", "Show"][
            "Show"
            in self.data_menu.GetLabel(self.data_menu_items[key].GetId())
        ]
        return show_hide

    def redraw_plots(self):
        if self.group_data[1]["dvh"]:
            self.plot.redraw_plot()
            self.time_series.plot.redraw_plot()
            self.correlation.plot.redraw_plot()
            self.regression.plot.redraw_plot()
            self.control_chart.plot.redraw_plot()

    def apply_plot_options(self):
        self.plot.apply_options()
        self.time_series.plot.apply_options()
        self.correlation.plot.apply_options()
        self.regression.apply_plot_options()  # applies to MVR frames too
        self.control_chart.plot.apply_options()

        for frame in self.loaded_mvr_ml_frames:
            try:
                frame.apply_plot_options()
            except RuntimeError:
                pass

    def update_stats_data_plots(self):
        if self.group_data[1]["dvh"]:
            self.time_series.update_plot()
            self.time_series.update_y_axis_options()
            self.regression.update_plot()
            self.control_chart.update_plot()

    def on_resize(self, *evt):
        try:
            if self.allow_window_size_save:
                self.options.set_window_size(self, "main")
            self.Refresh()
            self.Layout()
            wx.CallAfter(self.redraw_plots)
        except RuntimeError:
            pass

    def on_reset_windows(self, *evt):
        defaults = DefaultOptions()
        for key in ["MIN_RESOLUTION_MAIN", "MAX_INIT_RESOLUTION_MAIN"]:
            self.options.set_option(key, getattr(defaults, key))
        self.options.clear_positions()
        self.options.clear_window_sizes()
        self.__apply_size_and_position()

    def on_move(self, *evt):
        try:
            self.options.save_window_position(self, "main")
        except Exception:
            pass

    def on_plotting_memory_error(self, plot_type):
        plot_type = [" (Plot type: %s)" % plot_type, ""][plot_type is None]
        msg = (
            "Plotting memory error%s. Try querying less data. At least %s DVHs returned.\n"
            "NOTE: Threshold of this error is dependent on your computer."
            % (plot_type, self.group_data[1]["dvh"].count)
        )
        MemoryErrorDialog(self, msg)
        self.close()

    def correlation_patients_removed(self, msg):
        if not self.correlation_error_displayed:
            msg = (
                "These MRN(s) were removed from some or all of the Correlation calculations:\n%s"
                % ", ".join(list(msg["mrns"]))
            )
            caption = "Non-numerical data detected."
            wx.CallAfter(ErrorDialog, self, msg, caption)
            self.correlation_error_displayed = True

    def reset_query_filters(self):
        self.query_filters = {key: None for key in [1, 2]}

    def on_group_select(self, *evt):
        group = self.selected_group
        other = 3 - group

        self.button_query_execute.SetLabelText(
            "Query and Retrieve Group %s" % group
        )

        self.query_filters[other] = {
            "main_categorical": self.data_table_categorical.get_save_data(),
            "main_numerical": self.data_table_numerical.get_save_data(),
        }
        if self.query_filters[group] is not None:
            self.data_table_categorical.load_save_data(
                self.query_filters[group]["main_categorical"]
            )
            self.data_table_numerical.load_save_data(
                self.query_filters[group]["main_numerical"]
            )
        else:
            self.data_table_categorical.delete_all_rows()
            self.data_table_numerical.delete_all_rows()

        self.update_all_query_buttons()

        self.set_summary_text(group)

        if self.group_data[2]["stats_data"]:
            self.regression.group = group
            self.regression.update_plot()
            self.control_chart.group = group
            self.control_chart.update_plot()

    def set_summary_text(self, group):
        if self.group_data[group]["dvh"]:
            text = self.group_data[group]["dvh"].get_summary()
        else:
            text = ""
        self.text_summary.SetValue(text)

    def InstallThreadExcepthook(self):
        """Workaround for sys.excepthook thread bug from Jonathan Ellis
        (http://bugs.python.org/issue1230540).
        Call once from __main__ before creating any threads.
        If using psyco, call psyco.cannotcompile(threading.Thread.run)
        since this replaces a new-style class method."""

        run_old = threading.Thread.run

        def Run(*args, **kwargs):
            try:
                run_old(*args, **kwargs)
            except (KeyboardInterrupt, SystemExit):
                raise
            except Exception:
                sys.excepthook(*sys.exc_info())

        threading.Thread.run = Run


class MainApp(wx.App):
    def OnInit(self):

        initialize_directories()
        if is_windows():
            from dvha.tools.windows_reg_edit import (
                set_ie_emulation_level,
                set_ie_lockdown_level,
            )

            set_ie_emulation_level()
            set_ie_lockdown_level()

        self.SetAppName("DVH Analytics")
        self.frame = DVHAMainFrame(None, wx.ID_ANY, "")
        set_frame_icon(self.frame)
        self.SetTopWindow(self.frame)
        self.frame.Show()
        return True

    def OnExit(self):
        self.frame.options.save()
        for window in wx.GetTopLevelWindows():
            wx.CallAfter(window.Close)
        return super().OnExit()


def start():
    app = MainApp(0)
    app.MainLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# _version.py
"""Package initialization for DVHA-Analytics."""
# Copyright (c) 2020 Dan Cutright
# This file is part of DVH Analytics
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

__author__ = "Dan Cutright"
__email__ = "dan.cutright@gmail.com"
__version__ = "0.9.2.post2"
__release__ = "0.9.2.post2"
__version_info__ = (0, 9, 2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.database_editor.py
"""
Class for the Database Editor accessible from the main view
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics


import wx
from dvha.dialogs.database import (
    ChangePatientIdentifierDialog,
    DeletePatientDialog,
    ReimportDialog,
    EditDatabaseDialog,
    CalculationsDialog,
    DeleteAllData,
    RebuildDB,
    SQLErrorDialog,
)
from dvha.db.sql_to_python import get_database_tree
from dvha.db.sql_connector import DVH_SQL, SQLError
from dvha.models.data_table import DataTable
from dvha.dialogs.export import save_data_to_file
from dvha.tools.utilities import (
    set_msw_background_color,
    get_window_size,
    set_frame_icon,
)
from dvha.models.roi_map import RemapROIFrame
from dvha.tools.errors import ErrorDialog
from dvha.tools.threading_progress import ProgressFrame


class DatabaseEditorFrame(wx.Frame):
    """
    Various viewing and editing tools for the SQL database. This object is called on Database toolbar click.
    """

    def __init__(self, roi_map, options):
        """
        :param roi_map: roi_map object
        :type roi_map: DatabaseROIs
        """
        wx.Frame.__init__(self, None, title="Database Administrator")
        set_frame_icon(self)

        set_msw_background_color(
            self
        )  # If windows, change the background color

        self.roi_map = roi_map
        self.options = options
        self.db_tree = self.get_db_tree()

        self.SetSize(get_window_size(0.792, 0.781))

        self.window_db_editor = wx.SplitterWindow(
            self, wx.ID_ANY, style=wx.SP_3D
        )
        self.window_pane_db_tree = wx.ScrolledWindow(
            self.window_db_editor,
            wx.ID_ANY,
            style=wx.BORDER_SUNKEN | wx.TAB_TRAVERSAL,
        )
        self.tree_ctrl_db = wx.TreeCtrl(
            self.window_pane_db_tree,
            wx.ID_ANY,
            style=wx.TR_HAS_BUTTONS | wx.TR_MULTIPLE,
        )
        self.window_pane_query = wx.Panel(
            self.window_db_editor,
            wx.ID_ANY,
            style=wx.BORDER_SUNKEN | wx.TAB_TRAVERSAL,
        )
        self.text_ctrl_condition = wx.TextCtrl(
            self.window_pane_query, wx.ID_ANY, ""
        )
        self.list_ctrl_query_results = wx.ListCtrl(
            self.window_pane_query,
            wx.ID_ANY,
            style=wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES,
        )
        self.data_query_results = DataTable(
            self.list_ctrl_query_results, columns=["mrn", "study_instance_uid"]
        )
        self.combo_box_query_table = wx.ComboBox(
            self.window_pane_query,
            wx.ID_ANY,
            choices=list(self.db_tree),
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )

        self.button = {
            "delete_all_data": wx.Button(self, wx.ID_ANY, "Delete All Data"),
            "rebuild_db": wx.Button(self, wx.ID_ANY, "Rebuild Database"),
            "calculations": wx.Button(self, wx.ID_ANY, "Calculations"),
            "edit_db": wx.Button(self, wx.ID_ANY, "Edit Database"),
            "reimport": wx.Button(self, wx.ID_ANY, "Reimport from DICOM"),
            "delete_study": wx.Button(self, wx.ID_ANY, "Delete Study"),
            "change_mrn_uid": wx.Button(self, wx.ID_ANY, "Change MRN/UID"),
            "query": wx.Button(self.window_pane_query, wx.ID_ANY, "Query"),
            "clear": wx.Button(self.window_pane_query, wx.ID_ANY, "Clear"),
            "export_csv": wx.Button(
                self.window_pane_query, wx.ID_ANY, "Export"
            ),
            "remap_roi_names": wx.Button(self, wx.ID_ANY, "Remap ROI Names"),
            "update_from_csv": wx.Button(self, wx.ID_ANY, "Update from CSV"),
            # 'plan_complexity': wx.Button(self, wx.ID_ANY, "Recalc Plan Complexities"),
            "auto_fit_columns": wx.Button(
                self.window_pane_query, wx.ID_ANY, "Auto-fit Columns"
            ),
        }

        self.checkbox_auto_backup = wx.CheckBox(
            self, wx.ID_ANY, "Auto Backup SQLite DB After Import"
        )

        self.__set_properties()
        self.__do_layout()
        self.__do_bind()

        self.selected_columns = {
            table: {c: False for c in list(self.db_tree[table])}
            for table in list(self.db_tree)
        }
        self.selected_tables = {table: False for table in list(self.db_tree)}

        self.window_db_editor.SetSashPosition(250)
        self.tree_ctrl_db.Expand(self.db_tree_root)

        self.allow_tree_select_change = True

        self.Show()

    def __set_properties(self):
        self.SetTitle("Database Administrator")
        self.window_pane_db_tree.SetScrollRate(10, 10)
        self.window_db_editor.SetMinimumPaneSize(20)

        self.db_tree_root = self.tree_ctrl_db.AddRoot("DVH Analytics")
        self.table_nodes = {
            table: self.tree_ctrl_db.AppendItem(self.db_tree_root, table)
            for table in self.db_tree
        }
        self.column_nodes = {table: {} for table in self.db_tree}
        for table in self.db_tree:
            for column in self.db_tree[table]:
                self.column_nodes[table][
                    column
                ] = self.tree_ctrl_db.AppendItem(
                    self.table_nodes[table], column
                )
        self.combo_box_query_table.SetValue("Plans")

        self.checkbox_auto_backup.SetFont(
            wx.Font(
                11,
                wx.FONTFAMILY_DEFAULT,
                wx.FONTSTYLE_NORMAL,
                wx.FONTWEIGHT_NORMAL,
                0,
                "",
            )
        )
        self.checkbox_auto_backup.SetValue(self.options.AUTO_SQL_DB_BACKUP)

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_query = wx.BoxSizer(wx.VERTICAL)
        sizer_query_table = wx.StaticBoxSizer(
            wx.StaticBox(self.window_pane_query, wx.ID_ANY, "Query Results"),
            wx.VERTICAL,
        )
        sizer_condition_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_download_button = wx.BoxSizer(wx.VERTICAL)
        sizer_clear_button = wx.BoxSizer(wx.VERTICAL)
        sizer_query_button = wx.BoxSizer(wx.VERTICAL)
        sizer_auto_fit_columns_button = wx.BoxSizer(wx.VERTICAL)
        sizer_condition = wx.BoxSizer(wx.VERTICAL)
        sizer_combo_box = wx.BoxSizer(wx.VERTICAL)
        sizer_db_tree = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dialog_buttons = wx.BoxSizer(wx.VERTICAL)
        sizer_dialog_buttons_1 = wx.BoxSizer(wx.HORIZONTAL)

        sizer_dialog_buttons_1.Add(self.button["calculations"], 0, wx.ALL, 5)
        sizer_dialog_buttons_1.Add(self.button["edit_db"], 0, wx.ALL, 5)
        sizer_dialog_buttons_1.Add(self.button["reimport"], 0, wx.ALL, 5)
        sizer_dialog_buttons_1.Add(self.button["delete_study"], 0, wx.ALL, 5)
        sizer_dialog_buttons_1.Add(self.button["change_mrn_uid"], 0, wx.ALL, 5)
        sizer_dialog_buttons_1.Add(self.button["rebuild_db"], 0, wx.ALL, 5)
        sizer_dialog_buttons_1.Add(
            self.button["delete_all_data"], 0, wx.ALL, 5
        )
        sizer_dialog_buttons_1.Add(
            self.button["remap_roi_names"], 0, wx.ALL, 5
        )
        sizer_dialog_buttons_1.Add(
            self.button["update_from_csv"], 0, wx.ALL, 5
        )
        # sizer_dialog_buttons.Add(self.button['plan_complexity'], 0, wx.ALL, 5)
        sizer_dialog_buttons.Add(sizer_dialog_buttons_1, 0, 0, 0)
        sizer_dialog_buttons.Add(self.checkbox_auto_backup, 0, wx.LEFT, 5)
        sizer_wrapper.Add(sizer_dialog_buttons, 0, wx.ALL, 5)

        sizer_db_tree.Add(self.tree_ctrl_db, 1, wx.EXPAND, 0)
        self.window_pane_db_tree.SetSizer(sizer_db_tree)

        label_table = wx.StaticText(
            self.window_pane_query, wx.ID_ANY, "Table:"
        )
        sizer_combo_box.Add(label_table, 0, wx.BOTTOM | wx.TOP, 5)
        sizer_combo_box.Add(self.combo_box_query_table, 0, 0, 0)
        sizer_condition_buttons.Add(sizer_combo_box, 0, wx.ALL | wx.EXPAND, 5)

        label_condition = wx.StaticText(
            self.window_pane_query, wx.ID_ANY, "Condition:"
        )
        sizer_condition.Add(label_condition, 0, wx.BOTTOM | wx.TOP, 5)
        sizer_condition.Add(self.text_ctrl_condition, 0, wx.ALL | wx.EXPAND, 0)
        sizer_condition_buttons.Add(sizer_condition, 1, wx.ALL | wx.EXPAND, 5)

        label_spacer_1 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_query_button.Add(label_spacer_1, 0, wx.BOTTOM, 5)
        sizer_query_button.Add(self.button["query"], 0, wx.ALL, 5)
        sizer_condition_buttons.Add(sizer_query_button, 0, wx.ALL, 5)

        label_spacer_2 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_download_button.Add(label_spacer_2, 0, wx.BOTTOM, 5)
        sizer_download_button.Add(
            self.button["export_csv"], 0, wx.TOP | wx.BOTTOM, 5
        )
        sizer_condition_buttons.Add(sizer_download_button, 0, wx.ALL, 5)

        label_spacer_3 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_clear_button.Add(label_spacer_3, 0, wx.BOTTOM, 5)
        sizer_clear_button.Add(self.button["clear"], 0, wx.TOP | wx.BOTTOM, 5)
        sizer_condition_buttons.Add(sizer_clear_button, 0, wx.ALL, 5)

        label_spacer_4 = wx.StaticText(self.window_pane_query, wx.ID_ANY, "")
        sizer_auto_fit_columns_button.Add(label_spacer_4, 0, wx.BOTTOM, 5)
        sizer_auto_fit_columns_button.Add(
            self.button["auto_fit_columns"], 0, wx.ALL, 5
        )
        sizer_condition_buttons.Add(
            sizer_auto_fit_columns_button, 0, wx.ALL, 5
        )

        sizer_query.Add(
            sizer_condition_buttons,
            0,
            wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT,
            5,
        )

        sizer_query_table.Add(self.list_ctrl_query_results, 1, wx.EXPAND, 0)
        sizer_query.Add(sizer_query_table, 1, wx.ALL | wx.EXPAND, 5)

        self.window_pane_query.SetSizer(sizer_query)
        self.window_db_editor.SplitVertically(
            self.window_pane_db_tree, self.window_pane_query
        )
        sizer_wrapper.Add(self.window_db_editor, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_wrapper)
        self.Layout()
        self.Center()

    def __do_bind(self):
        # self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeAdd, self.tree_ctrl_db, id=self.tree_ctrl_db.GetId())

        # All buttons are bound to a function based on their key prepended with 'on_'
        # For example, query button calls on_query when clicked
        for key, button in self.button.items():
            self.Bind(
                wx.EVT_BUTTON, getattr(self, "on_" + key), id=button.GetId()
            )

        self.Bind(
            wx.EVT_CHECKBOX,
            self.on_auto_backup,
            id=self.checkbox_auto_backup.GetId(),
        )

        self.Bind(
            wx.EVT_LIST_COL_CLICK,
            self.sort_query_results,
            self.list_ctrl_query_results,
        )

    def on_auto_fit_columns(self, *evt):
        self.data_query_results.set_column_widths(auto=True)

    def on_tree_add(self, evt):
        self.update_selected_tree_items()

    # def UnselectOtherTables(self, selected_table):
    #     self.allow_tree_select_change = False
    #     for table, node in self.table_nodes.items():
    #         if table != selected_table:
    #             self.tree_ctrl_db.UnselectItem(node)
    #     self.allow_tree_select_change = True

    def on_query(self, evt):
        self.update_selected_tree_items()
        table = self.combo_box_query_table.GetValue()
        columns = [
            c
            for c, sel in self.selected_columns[table].items()
            if sel and c not in {"mrn", "study_instance_uid"}
        ]
        columns.sort()

        if not columns:
            columns = [
                c
                for c in self.db_tree[table]
                if c not in {"mrn", "study_instance_uid"}
            ]

        columns.insert(0, "study_instance_uid")
        columns.insert(0, "mrn")

        condition = self.text_ctrl_condition.GetValue()

        with DVH_SQL() as cnx:
            try:
                data = cnx.query(
                    table, ",".join(columns), condition, bokeh_cds=True
                )
                self.data_query_results.set_data(data, columns)
            except SQLError as e:
                SQLErrorDialog(self, e)
                self.data_query_results.clear()

    def on_clear(self, evt):
        self.data_query_results.clear()

    @staticmethod
    def get_db_tree():
        tree = get_database_tree()
        for table in list(tree):
            tree[table] = [
                column for column in tree[table] if "string" not in column
            ]
        return tree

    def update_selected_tree_items(self):
        for table, table_item in self.table_nodes.items():
            self.selected_tables[table] = self.tree_ctrl_db.IsSelected(
                table_item
            )
            for column, column_item in self.column_nodes[table].items():
                self.selected_columns[table][
                    column
                ] = self.tree_ctrl_db.IsSelected(column_item)

    def on_change_mrn_uid(self, evt):
        self.change_or_delete_dlg(ChangePatientIdentifierDialog)

    def on_delete_study(self, evt):
        self.change_or_delete_dlg(DeletePatientDialog)

    def change_or_delete_dlg(self, class_type):
        selected_data = self.data_query_results.selected_row_data
        if selected_data:
            class_type(
                mrn=selected_data[0][0], study_instance_uid=selected_data[0][1]
            )
        else:
            class_type()

    def on_reimport(self, evt):
        selected_data = self.data_query_results.selected_row_data
        if selected_data:
            ReimportDialog(
                self.roi_map,
                self.options,
                mrn=selected_data[0][0],
                study_instance_uid=selected_data[0][1],
            )
        else:
            ReimportDialog(self.roi_map, self.options)

    @staticmethod
    def on_edit_db(evt):
        EditDatabaseDialog()

    @staticmethod
    def on_calculations(evt):
        CalculationsDialog()

    def on_rebuild_db(self, evt):
        RebuildDB(self, self.roi_map, self.options)

    def on_delete_all_data(self, evt):
        DeleteAllData(self, self.options)

    def on_export_csv(self, evt):
        save_data_to_file(
            self, "Export Data Table to CSV", self.data_query_results.get_csv()
        )

    def on_remap_roi_names(self, evt):
        RemapROIFrame(self.roi_map, remap_all=True)

    def sort_query_results(self, evt):
        self.data_query_results.sort_table(evt)

    def on_auto_backup(self, *evt):
        self.options.AUTO_SQL_DB_BACKUP = self.checkbox_auto_backup.GetValue()

    # @staticmethod
    # def on_plan_complexity(*evt):
    #     with wx.BusyCursor() as busy:
    #         recalculate_plan_complexities_from_beams()

    def on_update_from_csv(self, *evt):
        msg = (
            "Load a CSV file and update your database. Please see manual for "
            "more information. Formatting is critical, this can not be undone!"
        )
        caption = "USE WITH EXTREME CAUTION"
        ErrorDialog(
            self, msg, caption, flags=wx.ICON_WARNING | wx.OK | wx.OK_DEFAULT
        )

        with wx.FileDialog(
            self,
            "Load CSV into Database",
            style=wx.FD_FILE_MUST_EXIST | wx.FD_OPEN,
        ) as fileDialog:
            if fileDialog.ShowModal() == wx.ID_CANCEL:
                return
            pathname = fileDialog.GetPath()
            with open(pathname, "r") as fp:
                lines = fp.readlines()
                header_row = "table,column,value,condition"
                if lines and header_row in lines[0].strip():
                    params = [csv_to_list(line.strip()) for line in lines[1:]]
                    ProgressFrame(
                        params,
                        update_db_with_csv,
                        title="Update Database from CSV file",
                        action_msg="Processing row",
                        star_map=True,
                    )
                else:
                    msg = (
                        "The selected file was not recognized. "
                        "Be sure you have a header row of: %s" % header_row
                    )
                    caption = "CSV load failure"
                    ErrorDialog(
                        self,
                        msg,
                        caption,
                        flags=wx.ICON_WARNING | wx.OK | wx.OK_DEFAULT,
                    )


def update_db_with_csv(*args):
    """Create SQL connection and call update"""
    with DVH_SQL() as cnx:
        cnx.update(*args)


def csv_to_list(csv_str, delimiter=","):
    """Split a CSV into a list

    Parameters
    ----------
    csv_str : str
        A comma-separated value string (with double quotes around values
        containing the delimiter)
    delimiter : str
        The str separator between values

    Returns
    ----------
    list
       csv_str split by the delimiter
    """
    if '"' not in csv_str:
        return csv_str.split(delimiter)

    # add an empty value with another ",", but ignore it
    # ensures next_csv_element always finds a ","
    next_value, csv_str = next_csv_element(csv_str + ",", delimiter)
    ans = [next_value.replace("<>", "\n")]
    while csv_str:
        next_value, csv_str = next_csv_element(csv_str, delimiter)
        ans.append(next_value.replace("<>", "\n"))

    return ans


def next_csv_element(csv_str, delimiter=","):
    """Helper function for csv_to_list

    Parameters
    ----------
    csv_str : str
        A comma-separated value string (with double quotes around values
        containing the delimiter)
    delimiter : str
        The str separator between values

    Returns
    ----------
    str, str
        Return a tuple, the next value and remainder of csv_str
    """
    if csv_str.startswith('"'):
        split = csv_str[1:].find('"') + 1
        return csv_str[1:split], csv_str[split + 2 :]

    next_delimiter = csv_str.find(delimiter)
    return csv_str[:next_delimiter], csv_str[next_delimiter + 1 :]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.time_series.py
"""
Class for viewing and editing the roi map, and updating the database with changes
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from dateutil.parser import parse as date_parser
from dvha.db import sql_columns
from dvha.dialogs.export import save_data_to_file
from dvha.models.plot import PlotTimeSeries


class TimeSeriesFrame:
    """
    Object to be passed into notebook panel for the Time Series tab
    """

    def __init__(self, main_app_frame):

        self.main_app_frame = main_app_frame
        self.parent = main_app_frame.notebook_tab["Time Series"]
        self.options = main_app_frame.options
        self.stats_data = {
            grp: data["stats_data"]
            for grp, data in main_app_frame.group_data.items()
        }

        self.combo_box_y_axis = wx.ComboBox(
            self.parent, wx.ID_ANY, style=wx.CB_DROPDOWN | wx.CB_READONLY
        )
        self.text_input_bin_size = wx.TextCtrl(
            self.parent, wx.ID_ANY, "10", style=wx.TE_PROCESS_ENTER
        )
        self.text_input_lookback_distance = wx.TextCtrl(
            self.parent, wx.ID_ANY, "1", style=wx.TE_PROCESS_ENTER
        )
        self.text_inputs_percentile = wx.TextCtrl(
            self.parent, wx.ID_ANY, "90", style=wx.TE_PROCESS_ENTER
        )
        self.button_update_plot = wx.Button(
            self.parent, wx.ID_ANY, "Update Plot"
        )
        self.button_export_csv = wx.Button(
            self.parent, wx.ID_ANY, "Export CSV"
        )
        self.button_save_figure = wx.Button(
            self.parent, wx.ID_ANY, "Save Figure"
        )

        self.parent.Bind(
            wx.EVT_COMBOBOX,
            self.combo_box_y_axis_ticker,
            id=self.combo_box_y_axis.GetId(),
        )
        self.parent.Bind(
            wx.EVT_TEXT_ENTER,
            self.update_plot_ticker,
            id=self.text_input_bin_size.GetId(),
        )
        self.parent.Bind(
            wx.EVT_TEXT_ENTER,
            self.update_plot_ticker,
            id=self.text_input_lookback_distance.GetId(),
        )
        self.parent.Bind(
            wx.EVT_TEXT_ENTER,
            self.update_plot_ticker,
            id=self.text_inputs_percentile.GetId(),
        )

        self.__set_properties()
        self.__do_layout()

        self.parent.Bind(
            wx.EVT_BUTTON,
            self.update_plot_ticker,
            id=self.button_update_plot.GetId(),
        )
        self.parent.Bind(
            wx.EVT_BUTTON, self.export_csv, id=self.button_export_csv.GetId()
        )
        self.parent.Bind(
            wx.EVT_BUTTON, self.save_figure, id=self.button_save_figure.GetId()
        )

        self.disable_buttons()

        self.layout_save_attr = [
            "combo_box_y_axis",
            "text_input_lookback_distance",
            "text_inputs_percentile",
            "text_input_bin_size",
        ]

    def __set_properties(self):
        self.combo_box_y_axis.AppendItems(self.choices)
        self.combo_box_y_axis.SetLabelText("ROI Max Dose")

    @property
    def choices(self):
        if self.stats_data[1]:
            choices = self.stats_data[1].variables
            if self.stats_data[2]:
                choices.extend(self.stats_data[2].variables)
                choices = list(set(choices))
        else:
            choices = list(sql_columns.numerical)
        choices.sort()
        return choices

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_plot = wx.BoxSizer(wx.HORIZONTAL)
        sizer_widgets = wx.StaticBoxSizer(
            wx.StaticBox(self.parent, wx.ID_ANY, ""), wx.VERTICAL
        )
        sizer_percentile = wx.BoxSizer(wx.VERTICAL)
        sizer_lookback_distance = wx.BoxSizer(wx.VERTICAL)
        sizer_histogram_bins = wx.BoxSizer(wx.VERTICAL)
        sizer_y_axis = wx.BoxSizer(wx.VERTICAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_input = wx.BoxSizer(wx.HORIZONTAL)

        label_y_axis = wx.StaticText(self.parent, wx.ID_ANY, "Y Axis:")
        sizer_y_axis.Add(label_y_axis, 0, wx.LEFT, 5)
        sizer_y_axis.Add(self.combo_box_y_axis, 0, wx.ALL | wx.EXPAND, 5)
        sizer_input.Add(sizer_y_axis, 1, wx.EXPAND, 0)

        label_histogram_bins = wx.StaticText(
            self.parent, wx.ID_ANY, "Hist. Bins:"
        )
        sizer_histogram_bins.Add(label_histogram_bins, 0, wx.LEFT, 5)
        sizer_histogram_bins.Add(
            self.text_input_bin_size, 0, wx.ALL | wx.EXPAND, 5
        )

        label_lookback_distance = wx.StaticText(
            self.parent, wx.ID_ANY, "Avg. Len:"
        )
        sizer_lookback_distance.Add(label_lookback_distance, 0, wx.LEFT, 5)
        sizer_lookback_distance.Add(
            self.text_input_lookback_distance, 0, wx.ALL | wx.EXPAND, 5
        )
        sizer_input.Add(sizer_lookback_distance, 1, wx.EXPAND, 0)

        label_percentile = wx.StaticText(self.parent, wx.ID_ANY, "Percentile:")
        sizer_percentile.Add(label_percentile, 0, wx.LEFT, 5)
        sizer_percentile.Add(
            self.text_inputs_percentile, 0, wx.ALL | wx.EXPAND, 5
        )
        sizer_input.Add(sizer_percentile, 1, wx.EXPAND, 0)
        sizer_input.Add(sizer_histogram_bins, 1, wx.EXPAND, 0)
        sizer_buttons.Add(self.button_update_plot, 0, wx.ALL | wx.EXPAND, 5)
        sizer_buttons.Add(self.button_export_csv, 0, wx.ALL | wx.EXPAND, 5)
        sizer_buttons.Add(self.button_save_figure, 0, wx.ALL | wx.EXPAND, 5)
        sizer_widgets.Add(sizer_input, 0, wx.ALL | wx.EXPAND, 0)
        sizer_widgets.Add(sizer_buttons, 0, wx.ALL | wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_widgets, 0, wx.BOTTOM | wx.EXPAND, 5)

        self.plot = PlotTimeSeries(self.parent, self.options)
        sizer_plot.Add(self.plot.layout, 1, wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_plot, 1, wx.EXPAND, 0)

        self.layout = sizer_wrapper

    def combo_box_y_axis_ticker(self, evt):
        self.update_plot()

    def update_plot(self):
        data = self.get_plot_data()
        self.plot.update_plot(data)

    def get_plot_data(self, y_axis_selection=None):
        if y_axis_selection is None:
            y_axis_selection = self.combo_box_y_axis.GetValue()
        data = {}
        for grp, stats_data in self.stats_data.items():
            if stats_data:
                y_data = stats_data.data[y_axis_selection]["values"]
                x_data = stats_data.sim_study_dates

                sort_index = sorted(
                    range(len(x_data)), key=lambda k: x_data[k]
                )
                x_values_sorted, y_values_sorted, mrn_sorted, uid_sorted = (
                    [],
                    [],
                    [],
                    [],
                )

                for s in range(len(x_data)):
                    try:
                        x = date_parser(x_data[sort_index[s]])
                    except Exception:
                        continue
                    x_values_sorted.append(x)
                    y_values_sorted.append(y_data[sort_index[s]])
                    mrn_sorted.append(stats_data.mrns[sort_index[s]])
                    uid_sorted.append(stats_data.uids[sort_index[s]])

                data[grp] = {
                    "x": x_values_sorted,
                    "y": y_values_sorted,
                    "mrn": mrn_sorted,
                    "uid": uid_sorted,
                    "y_axis_label": stats_data.get_axis_title(
                        y_axis_selection
                    ),
                    "avg_len": self.avg_len,
                    "percentile": self.percentile,
                    "bin_size": self.hist_bins,
                    "group": [grp] * len(x_values_sorted),
                }
            else:
                data[grp] = {
                    "x": [],
                    "y": [],
                    "mrn": [],
                    "uid": [],
                    "y_axis_label": [],
                    "avg_len": [],
                    "percentile": [],
                    "bin_size": [],
                    "group": [],
                }
        return data

    @property
    def hist_bins(self):
        try:
            hist_bins = int(self.text_input_bin_size.GetValue())
        except ValueError:
            self.text_input_bin_size.SetValue("10")
            hist_bins = 10
        return hist_bins

    @property
    def avg_len(self):
        try:
            avg_len = int(self.text_input_lookback_distance.GetValue())
        except ValueError:
            self.text_input_lookback_distance.SetValue("1")
            avg_len = 1
        return avg_len

    @property
    def percentile(self):
        try:
            percentile = float(self.text_inputs_percentile.GetValue())
        except ValueError:
            self.text_inputs_percentile.SetValue("90")
            percentile = 90.0
        return percentile

    def update_data(self, group_data):
        self.stats_data = {
            grp: data["stats_data"] for grp, data in group_data.items()
        }
        self.initialize_y_axis_options()
        self.update_plot()

    def clear_data(self):
        self.plot.clear_plot()
        self.initialize_y_axis_options()
        self.combo_box_y_axis.SetLabelText("ROI Max Dose")

    def enable_buttons(self):
        self.button_update_plot.Enable()

    def disable_buttons(self):
        self.button_update_plot.Disable()

    def enable_initial_buttons(self):
        self.button_update_plot.Enable()

    def update_plot_ticker(self, evt):
        self.update_plot()

    def update_y_axis_options(self):
        current_choice = self.combo_box_y_axis.GetValue()
        self.combo_box_y_axis.SetItems(self.choices)
        if current_choice not in self.choices:
            current_choice = "ROI Max Dose"
        self.combo_box_y_axis.SetValue(current_choice)

    def initialize_y_axis_options(self):
        self.combo_box_y_axis.SetItems(self.choices)
        self.combo_box_y_axis.SetValue("ROI Max Dose")

    def get_save_data(self):
        return {
            attr: getattr(self, attr).GetValue()
            for attr in self.layout_save_attr
        }

    def load_save_data(self, save_data):
        for attr in list(save_data):
            getattr(self, attr).SetValue(save_data[attr])
        self.update_y_axis_options()

    def get_csv(self, selection=None):
        """
        :param selection: variables to be included
        :type selection: list
        :return: csv data
        :rtype: str
        """

        # get_csv may be called from Time Series tab or DVHA app menu or tool bar
        # if selection is None, get_csv was called from Time Series tab, only export data in plot
        if selection is None:
            return self.plot.get_csv()

        # if selection is not None, export being called from DVHA app menu or tool bar

        csv = []
        for grp in [1, 2]:
            if self.stats_data[grp]:
                uids = self.stats_data[grp].uids
                mrns = self.stats_data[grp].mrns
                dates = self.stats_data[grp].sim_study_dates

                # Collect y-data (as in y-axis data from time series), organize into dict for printing to rows
                y_data = {}
                for y_axis in selection:
                    data = self.get_plot_data(y_axis_selection=y_axis)[grp]
                    column = []
                    for uid in uids:
                        if uid in data["uid"]:
                            index = data["uid"].index(uid)
                            column.append(data["y"][index])
                        else:
                            column.append("None")
                    y_data[y_axis] = column

                if grp == 2:
                    csv.insert(0, "Group 1")
                    csv.append("\nGroup 2")

                # Collect data into a list of row data
                csv.append(
                    "MRN,Study Instance UID,Date,%s" % ",".join(selection)
                )
                for i, uid in enumerate(uids):
                    row = [mrns[i], uid, str(dates[i])]
                    for y_axis in selection:
                        row.append(str(y_data[y_axis][i]))
                    csv.append(",".join(row))

        return "\n".join(csv)

    def export_csv(self, evt):
        save_data_to_file(
            self.parent, "Export Time Series data to CSV", self.plot.get_csv()
        )

    def save_figure(self, *evt):
        title = "Save Time Series Figure"
        export_frame = self.main_app_frame.export_figure
        attr_dicts = None if export_frame is None else export_frame.attr_dicts
        self.plot.save_figure_dlg(self.parent, title, attr_dicts=attr_dicts)

    @property
    def has_data(self):
        return self.button_export_csv.IsEnabled()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.endpoint.py
"""
Class for the Endpoint frame in the main view
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from copy import deepcopy
from dvha.models.data_table import DataTable
from dvha.dialogs.main import AddEndpointDialog, DelEndpointDialog
from dvha.dialogs.export import save_data_to_file
from dvha.tools.stats import sync_variables_in_stats_data_objects
from dvha.tools.utilities import get_window_size


class EndpointFrame:
    """
    Object to be passed into notebook panel for the Endpoint tab
    """

    def __init__(self, main_app_frame):

        self.main_app_frame = main_app_frame
        self.parent = main_app_frame.notebook_tab["Endpoints"]
        self.group_data = main_app_frame.group_data
        self.time_series = main_app_frame.time_series
        self.regression = main_app_frame.regression
        self.control_chart = main_app_frame.control_chart
        self.group_data = main_app_frame.group_data
        self.initial_columns = ["MRN", "Tx Site", "ROI Name", "Volume (cc)"]
        self.widths = [150, 150, 250, 100]

        self.button = {
            "add": wx.Button(self.parent, wx.ID_ANY, "Add Endpoint"),
            "del": wx.Button(self.parent, wx.ID_ANY, "Delete Endpoint"),
            "exp": wx.Button(self.parent, wx.ID_ANY, "Export"),
        }

        self.table = {
            key: wx.ListCtrl(
                self.parent,
                wx.ID_ANY,
                style=wx.BORDER_SUNKEN
                | wx.LC_HRULES
                | wx.LC_REPORT
                | wx.LC_VRULES,
            )
            for key in [1, 2]
        }
        for table in self.table.values():
            table.SetMinSize(get_window_size(0.623, 0.28))
        self.data_table = {key: DataTable(self.table[key]) for key in [1, 2]}

        self.endpoint_defs = DataTable(
            None,
            columns=[
                "label",
                "output_type",
                "input_type",
                "input_value",
                "units_in",
                "units_out",
            ],
        )

        for key in [1, 2]:
            if self.group_data[key]["dvh"]:
                self.group_data[key]["dvh"].endpoints[
                    "data"
                ] = self.data_table[key].data
                self.group_data[key]["dvh"].endpoints[
                    "defs"
                ] = self.endpoint_defs.data

        self.__do_bind()
        self.__set_properties()
        self.__do_layout()

        self.disable_buttons()

    def __do_bind(self):
        self.parent.Bind(
            wx.EVT_BUTTON,
            self.add_ep_button_click,
            id=self.button["add"].GetId(),
        )
        self.parent.Bind(
            wx.EVT_BUTTON,
            self.del_ep_button_click,
            id=self.button["del"].GetId(),
        )
        self.parent.Bind(
            wx.EVT_BUTTON, self.on_export_csv, id=self.button["exp"].GetId()
        )

    def __set_properties(self):
        for table in self.table.values():
            for i, column in enumerate(self.initial_columns):
                table.AppendColumn(
                    column, format=wx.LIST_FORMAT_LEFT, width=self.widths[i]
                )

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        vbox = wx.BoxSizer(wx.VERTICAL)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        for key in list(self.button):
            hbox.Add(self.button[key], 0, wx.ALL, 5)
        vbox.Add(hbox, 0, wx.ALL | wx.EXPAND, 5)
        for grp in [1, 2]:
            vbox.Add(
                wx.StaticText(self.parent, wx.ID_ANY, "Query Group %s:" % grp),
                0,
                wx.EXPAND | wx.BOTTOM,
                5,
            )
            vbox.Add(self.table[grp], 1, wx.EXPAND, 0)
            vbox.Add((10, 10), 0, 0, 0)
        sizer_wrapper.Add(vbox, 1, wx.ALL | wx.EXPAND, 20)
        self.layout = sizer_wrapper

    def calculate_endpoints(self):

        columns = {key: [c for c in self.initial_columns] for key in [1, 2]}
        if self.data_table[1].data:
            current_labels = [
                key
                for key in list(self.data_table[1].data)
                if key not in columns
            ]
        else:
            current_labels = []

        eps = {
            grp: {
                "MRN": group_data["dvh"].mrn,
                "Tx Site": group_data["dvh"].get_plan_values("tx_site"),
                "ROI Name": group_data["dvh"].roi_name,
                "Volume (cc)": group_data["dvh"].volume,
            }
            for grp, group_data in self.group_data.items()
            if group_data["dvh"]
        }

        ep_defs = self.endpoint_defs.data
        for group, ep in eps.items():
            if ep_defs:
                for i, ep_name in enumerate(ep_defs["label"]):

                    if ep_name not in columns[group]:
                        columns[group].append(ep_name)

                        if ep_name in current_labels:
                            ep[ep_name] = deepcopy(
                                self.data_table[group].data[ep_name]
                            )

                        else:
                            endpoint_input = ep_defs["input_type"][i]
                            endpoint_output = ep_defs["output_type"][i]

                            x = float(ep_defs["input_value"][i])
                            if endpoint_input == "relative":
                                x /= 100.0

                            dvh = self.group_data[group]["dvh"]
                            if "V" in ep_name:
                                ep[ep_name] = dvh.get_volume_of_dose(
                                    x,
                                    volume_scale=endpoint_output,
                                    dose_scale=endpoint_input,
                                )
                            else:
                                ep[ep_name] = dvh.get_dose_to_volume(
                                    x,
                                    dose_scale=endpoint_output,
                                    volume_scale=endpoint_input,
                                )

        for group, ep in eps.items():
            self.data_table[group].set_data(ep, columns[group])
            self.data_table[group].set_column_width(0, 150)
            self.data_table[group].set_column_width(1, 150)
            self.data_table[group].set_column_width(2, 200)

    def add_ep_button_click(self, evt):
        # TODO: Track down duplicate table refreshes/calculations
        dlg = AddEndpointDialog()
        res = dlg.ShowModal()
        if res == wx.ID_OK and dlg.is_endpoint_valid:
            self.endpoint_defs.append_row(dlg.endpoint_row)
            self.calculate_endpoints()
            self.enable_buttons()
            self.update_endpoints_in_dvh()
        dlg.Destroy()
        self.update_endpoints_and_radbio_in_group_data()
        self.regression.update_combo_box_choices()
        self.control_chart.update_combo_box_y_choices()
        self.time_series.update_y_axis_options()

    def del_ep_button_click(self, evt):
        dlg = DelEndpointDialog(self.data_table[1].columns)
        res = dlg.ShowModal()
        if res == wx.ID_OK:
            for value in dlg.selected_values:
                for data_table in self.data_table.values():
                    if data_table.columns and value in data_table.columns:
                        data_table.delete_column(value)
                endpoint_def_row = self.endpoint_defs.data["label"].index(
                    value
                )
                self.update_endpoints_in_dvh()
                self.endpoint_defs.delete_row(endpoint_def_row)
                self.update_endpoints_and_radbio_in_group_data()

            self.time_series.update_y_axis_options()
        dlg.Destroy()

        for group in self.group_data.values():
            if group["stats_data"]:
                group["stats_data"].update_endpoints_and_radbio()
        self.regression.update_combo_box_choices()
        self.control_chart.update_combo_box_y_choices()

        for data_table in self.data_table.values():
            if data_table.column_count == 3:
                self.button["del"].Disable()
                self.button["exp"].Disable()

    def update_dvh(self, group_data):
        self.group_data = group_data
        self.update_endpoints_in_dvh()

    def update_endpoints_and_radbio_in_group_data(self):
        for grp, data in self.group_data.items():
            if data["stats_data"]:
                data["stats_data"].update_endpoints_and_radbio()
                if grp == 2:
                    sync_variables_in_stats_data_objects(
                        self.group_data[1]["stats_data"],
                        self.group_data[2]["stats_data"],
                    )

    def update_endpoints_in_dvh(self):
        for group in [1, 2]:
            if self.group_data[group]["dvh"]:
                self.group_data[group]["dvh"].endpoints[
                    "data"
                ] = self.data_table[group].data
                self.group_data[group]["dvh"].endpoints[
                    "defs"
                ] = self.endpoint_defs.data

    def clear_data(self):
        for data_table in self.data_table.values():
            data_table.delete_all_rows()
        self.endpoint_defs.delete_all_rows(
            force_delete_data=True
        )  # no attached layout, force delete

        for data_table in self.data_table.values():
            if data_table.data:
                for column in list(data_table.data):
                    if column not in self.initial_columns:
                        data_table.delete_column(column)

    def enable_buttons(self):
        for key in list(self.button):
            self.button[key].Enable()

    def disable_buttons(self):
        for key in list(self.button):
            self.button[key].Disable()

    def enable_initial_buttons(self):
        self.button["add"].Enable()

    def get_csv(self, selection=None):
        uid = {
            1: {
                "title": "Study Instance UID",
                "data": self.group_data[1]["dvh"].uid,
            }
        }
        csv = self.data_table[1].get_csv(extra_column_data=uid)
        if self.group_data[2]["dvh"]:
            uid = {
                1: {
                    "title": "Study Instance UID",
                    "data": self.group_data[2]["dvh"].uid,
                }
            }
            csv = "Group 1\n%s\n\nGroup 2\n%s" % (
                csv,
                self.data_table[2].get_csv(extra_column_data=uid),
            )

        return csv

    def on_export_csv(self, evt):
        save_data_to_file(
            self.parent, "Export Endpoints to CSV", self.get_csv()
        )

    def get_save_data(self):
        return {
            "data_table_1": self.data_table[1].get_save_data(),
            "data_table_2": self.data_table[2].get_save_data(),
            "endpoint_defs": self.endpoint_defs.get_save_data(),
        }

    def load_save_data(self, save_data):
        self.data_table[1].load_save_data(
            save_data["data_table_1"], ignore_layout=True
        )
        self.data_table[2].load_save_data(
            save_data["data_table_2"], ignore_layout=True
        )
        self.endpoint_defs.load_save_data(
            save_data["endpoint_defs"], ignore_layout=True
        )
        self.calculate_endpoints()

    @property
    def has_data(self):
        if self.endpoint_defs.data and self.endpoint_defs.data["label"]:
            return True
        return False
